#!/usr/bin/python
#
# Copyright 2012 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Sample to delete suspended users whose last login date is 6 months old.

Sample to obtain the suspended users and their last login date from the
Reporting API and delete the user if the last login date is older than
6 months using the Provisioning API.
"""

__author__ = 'Shraddha Gupta <shraddhag@google.com>'

import datetime
import getopt
import sys
import gdata.apps.client
import gdata.client
import google.apps.reporting


def Usage():
  print ('Usage: suspended_users_cleanup.py --email=<email> '
         '--password=<password> --domain=<domain> ')


def GetAccountsReport(report_runner):
  """Gets the account report from the Reporting API for current date.

  Args:
    report_runner: An instance of google.apps.reporting.ReportRunner.

  Returns:
    response: String containing accounts report.
  """
  if report_runner.token is None:
    report_runner.Login()
  request = google.apps.reporting.ReportRequest()
  request.token = report_runner.token
  request.domain = report_runner.domain
  request.report_name = 'accounts'
  request.date = report_runner.GetLatestReportDate()
  response = report_runner.GetReportData(request)
  return response


def main():
  """Gets accounts report and deletes old suspended users."""
  try:
    (opts, args) = getopt.getopt(sys.argv[1:], '', ['email=', 'password=',
                                                    'domain='])
  except getopt.GetoptError:
    print 'Error'
    Usage()
    sys.exit(1)
  opts = dict(opts)

  report_runner = google.apps.reporting.ReportRunner()
  email = opts.get('--email')
  report_runner.admin_email = email
  password = opts.get('--password')
  report_runner.admin_password = password
  domain = opts.get('--domain')
  report_runner.domain = domain
  if not email or not password or not domain:
    Usage()
    sys.exit(1)

  # Instantiate a client for Provisioning API
  client = gdata.apps.client.AppsClient(domain=domain)
  client.ClientLogin(email=email, password=password,
                     source='DeleteOldSuspendedUsers')

  # Get accounts report from the Reporting API
  response = GetAccountsReport(report_runner)
  accounts = response.splitlines()
  current_date = datetime.datetime.now()

  # accounts[0] contains the headings for fields, read data starting at index 1
  for i in range(1, len(accounts)):
    account_fields = accounts[i].split(',')
    # Find the suspended users and check if last login date is 180 days old
    if account_fields[3] != 'ACTIVE':
      last_login_date_str = account_fields[10]
      last_login_date = datetime.datetime.strptime(last_login_date_str,
                                                   '%Y%m%d')
      duration = current_date - last_login_date
      days = duration.days
      if days > 180:
        account_id = account_fields[2]
        print 'This user is obsolete: %s' % account_id
        user_name = account_id.split('@')
        delete_flag = raw_input('Do you want to delete the user (y/n)? ')
        if delete_flag == 'y':
          try:
            client.DeleteUser(user_name[0])
            print 'Deleted %s ' % account_id
          except gdata.client.RequestError, e:
            print 'Request Error %s %s %s' % (e.status, e.reason, e.body)


if __name__ == '__main__':
  main()



#!/usr/bin/python2.4
#
# Copyright 2011 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Sample app for Google Apps Provisioning API using OAuth.

  ProvisioningOAuthSample: Demonstrates the use of the Provisioning API with
  OAuth in a Google App Engine app.
"""


__author__ = 'pti@google.com (Prashant Tiwari)'


import json
import os
import atom.http_interface
import gdata.apps.service
import gdata.auth
from django.utils import simplejson
from google.appengine.ext import webapp
from google.appengine.ext.webapp import template
from google.appengine.ext.webapp.util import run_wsgi_app
from gdata.apps.service import AppsForYourDomainException


INIT = {
    'APP_NAME': 'googlecode-provisioningtester-v1',
    'SCOPES': ['https://apps-apis.google.com/a/feeds']
}

secret = None

class MainPage(webapp.RequestHandler):
  """Defines the entry point for the App Engine app"""
  def get(self):
    global service
    try:
      service
    except:
      self.redirect('/login')
      return

    oauth_token = None
    json = None
    if self.request.get('two_legged_oauth'):
      two_legged_oauth = True
      app_title = 'Provisioning API Sample (2-legged OAuth)'
      start_over_text = 'Start over'
    else:
      two_legged_oauth = False
      app_title = 'Provisioning API Sample (3-legged OAuth)'
      start_over_text = 'Revoke token'

    try:
      if service:
        oauth_token = service.token_store.find_token('%20'.join(INIT['SCOPES']))
        if isinstance(oauth_token, gdata.auth.OAuthToken) or isinstance(
            oauth_token, atom.http_interface.GenericToken):
          user_feed = service.RetrieveAllUsers()
          json = get_json_from_feed(user_feed)
      else:
        self.redirect('/login')
        return
    except AppsForYourDomainException, e:
      # Usually a Forbidden (403) when signed-in user isn't the admin.
      self.response.out.write(e.args[0].get('body'))
    else:
      template_values = {
          'oauth_token': oauth_token,
          'json': json,
          'two_legged_oauth': two_legged_oauth,
          'start_over_text': start_over_text,
          'app_title': app_title
      }

      path = os.path.join(os.path.dirname(__file__), 'index.html')
      self.response.out.write(template.render(path, template_values))


def get_json_from_feed(user_feed):
  """Constructs and returns a JSON object from the given feed object
  
  Args:
    user_feed: A gdata.apps.UserFeed object
  
  Returns:
    A JSON object containing the first and last names of the domain users
  """
  json = []
  for entry in user_feed.entry:
    json.append({'given_name': entry.name.given_name,
                 'family_name': entry.name.family_name,
                 'username': entry.login.user_name,
                 'admin': entry.login.admin
                })
  return simplejson.dumps(json)


class DoLogin(webapp.RequestHandler):
  """Brings up the app's login page"""
  def get(self):
    path = os.path.join(os.path.dirname(__file__), 'login.html')
    self.response.out.write(template.render(path, None))


class DoAuth(webapp.RequestHandler):
  """Handles the entire OAuth flow for the app"""
  def post(self):
    global service
    global secret
    # Get instance of the AppsService for the given consumer_key (domain)
    service = gdata.apps.service.AppsService(source=INIT['APP_NAME'],
                                             domain=self.request.get('key'))
    two_legged_oauth = False
    if self.request.get('oauth') == 'two_legged_oauth':
      two_legged_oauth = True
    service.SetOAuthInputParameters(
        signature_method=gdata.auth.OAuthSignatureMethod.HMAC_SHA1,
        consumer_key=self.request.get('key'),
        consumer_secret=self.request.get('secret'),
        two_legged_oauth=two_legged_oauth)

    if two_legged_oauth:
      # Redirect to MainPage if 2-legged OAuth is requested
      self.redirect('/?two_legged_oauth=true')
      return

    request_token = service.FetchOAuthRequestToken(
        scopes=INIT['SCOPES'], oauth_callback=self.request.uri)
    secret = request_token.secret
    service.SetOAuthToken(request_token)
    # Send user to Google authorization page
    google_auth_page_url = service.GenerateOAuthAuthorizationURL()
    self.redirect(google_auth_page_url)

  def get(self):
    global service
    global secret
    # Extract the OAuth request token from the URL
    oauth_token = gdata.auth.OAuthTokenFromUrl(self.request.uri)
    if oauth_token:
      oauth_token.secret = secret
      oauth_token.oauth_input_params = service.GetOAuthInputParameters()
      service.SetOAuthToken(oauth_token)
      # Exchange the request token for an access token
      oauth_verifier = self.request.get('oauth_verifier', default_value='')
      access_token = service.UpgradeToOAuthAccessToken(
          oauth_verifier=oauth_verifier)
      # Store access_token to the service token_store for later access
      if access_token:
        service.current_token = access_token
        service.SetOAuthToken(access_token)

    self.redirect('/')


class DoStartOver(webapp.RequestHandler):
  """Revokes the OAuth token if needed and starts over"""
  def get(self):
    global service
    two_legged_oauth = self.request.get('two_legged_oauth')
    # Revoke the token for 3-legged OAuth
    if two_legged_oauth != 'True':
      try:
        service.RevokeOAuthToken()
      except gdata.service.RevokingOAuthTokenFailed:
        pass
      except gdata.service.NonOAuthToken:
        pass
      finally:
        service.token_store.remove_all_tokens()

    service = None
    self.redirect('/')


application = webapp.WSGIApplication([('/', MainPage),
                                      ('/do_auth', DoAuth),
                                      ('/start_over', DoStartOver),
                                      ('/login', DoLogin)],
                                      debug=True)


def main():
    run_wsgi_app(application)

if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright 2009 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Samples for the Documents List API v3."""

__author__ = 'afshar@google.com (Ali Afshar)'

import os.path
import gdata.data
import gdata.acl.data
import gdata.docs.client
import gdata.docs.data
import gdata.sample_util


class SampleConfig(object):
  APP_NAME = 'GDataDocumentsListAPISample-v1.0'
  DEBUG = False


def CreateClient():
  """Create a Documents List Client."""
  client = gdata.docs.client.DocsClient(source=SampleConfig.APP_NAME)
  client.http_client.debug = SampleConfig.DEBUG
  # Authenticate the user with CLientLogin, OAuth, or AuthSub.
  try:
    gdata.sample_util.authorize_client(
        client,
        service=client.auth_service,
        source=client.source,
        scopes=client.auth_scopes
    )
  except gdata.client.BadAuthentication:
    exit('Invalid user credentials given.')
  except gdata.client.Error:
    exit('Login Error')
  return client


def PrintResource(resource):
  """Display a resource to Standard Out."""
  print resource.resource_id.text, resource.GetResourceType()


def PrintFeed(feed):
  """Display a feed to Standard Out."""
  for entry in feed.entry:
    PrintResource(entry)


def _GetDataFilePath(name):
  return os.path.join(
      os.path.dirname(
          os.path.dirname(
              os.path.dirname(__file__))),
      'tests', 'gdata_tests', 'docs', 'data', name)


def GetResourcesSample():
  """Get and display first page of resources."""
  client = CreateClient()
  # Get a feed and print it
  feed = client.GetResources()
  PrintFeed(feed)


def GetAllResourcesSample():
  """Get and display all resources, using pagination."""
  client = CreateClient()
  # Unlike client.GetResources, this returns a list of resources
  for resource in client.GetAllResources():
    PrintResource(resource)


def GetResourceSample():
  """Fetch 5 resources from a feed, then again individually."""
  client = CreateClient()
  for e1 in client.GetResources(limit=5).entry:
    e2 = client.GetResource(e1)
    print 'Refetched: ', e2.title.text, e2.resource_id.text


def GetMetadataSample():
  """Get and display the Metadata for the current user."""
  client = CreateClient()
  # Fetch the metadata entry and display bits of it
  metadata = client.GetMetadata()
  print 'Quota'
  print '  Total:', metadata.quota_bytes_total.text
  print '  Used:', metadata.quota_bytes_used.text
  print '  Trashed:', metadata.quota_bytes_used_in_trash.text
  print 'Import / Export'
  for input_format in metadata.import_formats:
    print '  Import:', input_format.source, 'to', input_format.target
  for export_format in metadata.export_formats:
    print '  Export:', export_format.source, 'to', export_format.target
  print 'Features'
  for feature in metadata.features:
    print '  Feature:', feature.name.text
  print 'Upload Sizes'
  for upload_size in metadata.max_upload_sizes:
    print '  Kind:', upload_size.kind, upload_size.text


def GetChangesSample():
  """Get and display the Changes for the user."""
  client = CreateClient()
  changes = client.GetChanges()
  for change in changes.entry:
    print change.title.text, change.changestamp.value


def GetResourceAclSample():
  """Get and display the ACL for a resource."""
  client = CreateClient()
  for resource in client.GetResources(limit=5).entry:
    acl_feed = client.GetResourceAcl(resource)
    for acl in acl_feed.entry:
      print acl.role.value, acl.scope.type, acl.scope.value


def CreateEmptyResourceSample():
  """Create an empty resource of type document."""
  client = CreateClient()
  document = gdata.docs.data.Resource(type='document', title='My Sample Doc')
  document = client.CreateResource(document)
  print 'Created:', document.title.text, document.resource_id.text


def CreateCollectionSample():
  """Create an empty collection."""
  client = CreateClient()
  col = gdata.docs.data.Resource(type='folder', title='My Sample Folder')
  col = client.CreateResource(col)
  print 'Created collection:', col.title.text, col.resource_id.text


def CreateResourceInCollectionSample():
  """Create a collection, then create a document in it."""
  client = CreateClient()
  col = gdata.docs.data.Resource(type='folder', title='My Sample Folder')
  col = client.CreateResource(col)
  print 'Created collection:', col.title.text, col.resource_id.text
  doc = gdata.docs.data.Resource(type='document', title='My Sample Doc')
  doc = client.CreateResource(doc, collection=col)
  print 'Created:', doc.title.text, doc.resource_id.text


def UploadResourceSample():
  """Upload a document, and convert to Google Docs."""
  client = CreateClient()
  doc = gdata.docs.data.Resource(type='document', title='My Sample Doc')
  # Set the description
  doc.description = gdata.docs.data.Description(
      'This is a simple Word document.')
  # This is a convenient MS Word doc that we know exists
  path = _GetDataFilePath('test.0.doc')
  print 'Selected file at: %s' % path
  # Create a MediaSource, pointing to the file
  media = gdata.data.MediaSource()
  media.SetFileHandle(path, 'application/msword')
  # Pass the MediaSource when creating the new Resource
  doc = client.CreateResource(doc, media=media)
  print 'Created, and uploaded:', doc.title.text, doc.resource_id.text


def UploadUnconvertedFileSample():
  """Upload a document, unconverted."""
  client = CreateClient()
  doc = gdata.docs.data.Resource(type='document', title='My Sample Raw Doc')
  path = _GetDataFilePath('test.0.doc')
  media = gdata.data.MediaSource()
  media.SetFileHandle(path, 'application/msword')
  # Pass the convert=false parameter
  create_uri = gdata.docs.client.RESOURCE_UPLOAD_URI + '?convert=false'
  doc = client.CreateResource(doc, create_uri=create_uri, media=media)
  print 'Created, and uploaded:', doc.title.text, doc.resource_id.text


def DeleteResourceSample():
  """Delete a resource (after creating it)."""
  client = CreateClient()
  doc = gdata.docs.data.Resource(type='document', title='My Sample Doc')
  doc = client.CreateResource(doc)
  # Delete the resource we just created.
  client.DeleteResource(doc)


def AddAclSample():
  """Create a resource and an ACL."""
  client = CreateClient()
  doc = gdata.docs.data.Resource(type='document', title='My Sample Doc')
  doc = client.CreateResource(doc)
  acl_entry = gdata.docs.data.AclEntry(
      scope=gdata.acl.data.AclScope(value='user@example.com', type='user'),
      role=gdata.acl.data.AclRole(value='reader'),
  )
  client.AddAclEntry(doc, acl_entry, send_notifications=False)


def DeleteAclSample():
  """Create an ACL entry, and delete it."""
  client = CreateClient()
  doc = gdata.docs.data.Resource(type='document', title='My Sample Doc')
  doc = client.CreateResource(doc)
  acl_entry = gdata.docs.data.AclEntry(
      scope=gdata.acl.data.AclScope(value='user@example.com', type='user'),
      role=gdata.acl.data.AclRole(value='reader'),
  )
  acl_entry = client.AddAclEntry(doc, acl_entry)
  client.DeleteAclEntry(acl_entry)


def AddAclBatchSample():
  """Add a list of ACLs as a batch."""
  client = CreateClient()
  doc = gdata.docs.data.Resource(type='document', title='My Sample Doc')
  doc = client.CreateResource(doc)
  acl1 = gdata.docs.data.AclEntry(
      scope=gdata.acl.data.AclScope(value='user1@example.com', type='user'),
      role=gdata.acl.data.AclRole(value='reader'),
      batch_operation=gdata.data.BatchOperation(type='insert'),
  )
  acl2 = gdata.docs.data.AclEntry(
      scope=gdata.acl.data.AclScope(value='user2@example.com', type='user'),
      role=gdata.acl.data.AclRole(value='reader'),
      batch_operation=gdata.data.BatchOperation(type='insert'),
  )
  # Create a list of operations to perform together.
  acl_operations = [acl1, acl2]
  # Perform the operations.
  client.BatchProcessAclEntries(doc, acl_operations)


def GetRevisionsSample():
  """Get the revision history for resources."""
  client = CreateClient()
  for entry in client.GetResources(limit=55).entry:
    revisions = client.GetRevisions(entry)
    for revision in revisions.entry:
      print revision.publish, revision.GetPublishLink()


if __name__ == '__main__':
  import samplerunner
  samplerunner.Run(__file__)



"""
Copyright (c) 2008, appengine-utilities project
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
- Redistributions of source code must retain the above copyright notice, this
  list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice,
  this list of conditions and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
- Neither the name of the appengine-utilities project nor the names of its
  contributors may be used to endorse or promote products derived from this
  software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""

__author__="jbowman"
__date__ ="$Sep 11, 2009 4:20:11 PM$"


# Configuration settings for the session class.
session = {    
    "COOKIE_NAME": "gaeutilities_session",
    "DEFAULT_COOKIE_PATH": "/",
    "DEFAULT_COOKIE_DOMAIN": False, # Set to False if you do not want this value
                                    # set on the cookie, otherwise put the
                                    # domain value you wish used.
    "SESSION_EXPIRE_TIME": 7200,    # sessions are valid for 7200 seconds
                                    # (2 hours)
    "INTEGRATE_FLASH": True,        # integrate functionality from flash module?
    "SET_COOKIE_EXPIRES": True,     # Set to True to add expiration field to
                                    # cookie
    "WRITER":"datastore",           # Use the datastore writer by default. 
                                    # cookie is the other option.
    "CLEAN_CHECK_PERCENT": 50,      # By default, 50% of all requests will clean
                                    # the datastore of expired sessions
    "CHECK_IP": True,               # validate sessions by IP
    "CHECK_USER_AGENT": True,       # validate sessions by user agent
    "SESSION_TOKEN_TTL": 5,         # Number of seconds a session token is valid
                                    # for.
    "UPDATE_LAST_ACTIVITY": 60,     # Number of seconds that may pass before
                                    # last_activity is updated
}

# Configuration settings for the cache class
cache = {
    "DEFAULT_TIMEOUT": 3600, # cache expires after one hour (3600 sec)
    "CLEAN_CHECK_PERCENT": 50, # 50% of all requests will clean the database
    "MAX_HITS_TO_CLEAN": 20, # the maximum number of cache hits to clean
}

# Configuration settings for the flash class
flash = {
    "COOKIE_NAME": "appengine-utilities-flash",
}

# Configuration settings for the paginator class
paginator = {
    "DEFAULT_COUNT": 10,
    "CACHE": 10,
    "DEFAULT_SORT_ORDER": "ASC",
}

rotmodel = {
    "RETRY_ATTEMPTS": 3,
    "RETRY_INTERVAL": .2,
}
if __name__ == "__main__":
    print "Hello World";




#!/usr/bin/python

import sys
import unittest
import module_test_runner
import getopt
import getpass

# Modules whose tests we will run.
import atom_tests.service_test
import gdata_tests.service_test
import gdata_tests.apps.service_test
import gdata_tests.books.service_test
import gdata_tests.calendar.service_test
import gdata_tests.docs.service_test
import gdata_tests.health.service_test
import gdata_tests.spreadsheet.service_test
import gdata_tests.spreadsheet.text_db_test
import gdata_tests.photos.service_test
import gdata_tests.contacts.service_test
import gdata_tests.blogger.service_test
import gdata_tests.youtube.service_test
import gdata_tests.health.service_test
import gdata_tests.contacts.profiles.service_test


def RunAllTests(username, password, spreadsheet_key, worksheet_key,
                apps_username, apps_password, apps_domain):
  test_runner = module_test_runner.ModuleTestRunner()
  test_runner.modules = [atom_tests.service_test, 
                         gdata_tests.service_test, 
                         gdata_tests.apps.service_test,
                         gdata_tests.base.service_test, 
                         gdata_tests.books.service_test, 
                         gdata_tests.calendar.service_test,
                         gdata_tests.docs.service_test, 
                         gdata_tests.health.service_test,
                         gdata_tests.spreadsheet.service_test,
                         gdata_tests.spreadsheet.text_db_test,
                         gdata_tests.contacts.service_test,
                         gdata_tests.youtube.service_test,
                         gdata_tests.photos.service_test,
                         gdata_tests.contacts.profiles.service_test,]
  test_runner.settings = {'username':username, 'password':password, 
                          'test_image_location':'testimage.jpg',
                          'ss_key':spreadsheet_key,
                          'ws_key':worksheet_key,
                          'apps_username':apps_username,
                          'apps_password':apps_password,
                          'apps_domain':apps_domain}
  test_runner.RunAllTests()

def GetValuesForTestSettingsAndRunAllTests():
  username = ''
  password = ''
  spreadsheet_key = ''
  worksheet_key = ''
  apps_domain = ''
  apps_username = ''
  apps_password = ''
  
  print ('NOTE: Please run these tests only with a test account. ' 
         'The tests may delete or update your data.')
  try:
    opts, args = getopt.getopt(sys.argv[1:], '', ['username=', 'password=',
                                                  'ss_key=', 'ws_key=',
                                                  'apps_username=',
                                                  'apps_password=',
                                                  'apps_domain='])
    for o, a in opts:
      if o == '--username':
        username = a
      elif o == '--password':
        password = a
      elif o == '--ss_key':
        spreadsheet_key = a
      elif o == '--ws_key':
        worksheet_key = a
      elif o == '--apps_username':
        apps_username = a
      elif o == '--apps_password':
        apps_password = a
      elif o == '--apps_domain':
        apps_domain = a
  except getopt.GetoptError:
    pass

  if username == '' and password == '':
    print ('Missing --user and --pw command line arguments, '  
           'prompting for credentials.')
  if username == '':
    username = raw_input('Please enter your username: ')
  if password == '':
    password = getpass.getpass()
  if spreadsheet_key == '':
    spreadsheet_key = raw_input(
        'Please enter the key for the test spreadsheet: ')
  if worksheet_key == '':
    worksheet_key = raw_input(
        'Please enter the id for the worksheet to be edited: ')
  if apps_username == '':
    apps_username = raw_input('Please enter your Google Apps admin username: ')
  if apps_password == '':
    apps_password = getpass.getpass()
  if apps_domain == '':
    apps_domain = raw_input('Please enter your Google Apps domain: ')
  RunAllTests(username, password, spreadsheet_key, worksheet_key,
              apps_username, apps_password, apps_domain)

if __name__ == '__main__':
  GetValuesForTestSettingsAndRunAllTests()



#!/usr/bin/python
#
# Copyright (C) 2007 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

__author__ = 'api.lliabraa@google.com (Lane LiaBraaten)'

import unittest
try:
  from xml.etree import ElementTree
except ImportError:
  from elementtree import ElementTree
import atom
import gdata.calendar
import gdata.calendar.service
import gdata.service
import random
import getpass
from gdata import test_data

username = ''
password = ''

class CalendarServiceAclUnitTest(unittest.TestCase):
  _aclFeedUri = "/calendar/feeds/default/acl/full"
  _aclEntryUri = "%s/user:%s" % (_aclFeedUri, "user@gmail.com",)
  
  def setUp(self):
    self.cal_client = gdata.calendar.service.CalendarService()
    self.cal_client.email = username 
    self.cal_client.password = password
    self.cal_client.source = 'GCalendarClient ACL "Unit" Tests'

  def tearDown(self):
    # No teardown needed
    pass  

  def _getRandomNumber(self):
    """Return a random number as a string for testing"""
    r = random.Random()
    r.seed()
    return str(r.randint(100000,1000000))

  def _generateAclEntry(self, role="owner", scope_type="user", scope_value=None):
    """Generates a ACL rule from parameters or makes a random user an owner by default"""
    if (scope_type=="user" and scope_value is None):
      scope_value = "user%s@gmail.com" % (self._getRandomNumber())
    rule = gdata.calendar.CalendarAclEntry()
    rule.title = atom.Title(text=role)
    rule.scope = gdata.calendar.Scope(value=scope_value, type="user")
    rule.role = gdata.calendar.Role(value="http://schemas.google.com/gCal/2005#%s" % (role))
    return rule

  def assertEqualAclEntry(self, expected, actual):
    """Compares the values of two ACL entries"""
    self.assertEqual(expected.role.value, actual.role.value)
    self.assertEqual(expected.scope.value, actual.scope.value)
    self.assertEqual(expected.scope.type, actual.scope.type)

  def testGetAclFeedUnauthenticated(self):
    """Fiendishly try to get an ACL feed without authenticating"""
    try:
      self.cal_client.GetCalendarAclFeed(self._aclFeedUri)
      self.fail("Unauthenticated request should fail")
    except gdata.service.RequestError, error:
      self.assertEqual(error[0]['status'], 401)
      self.assertEqual(error[0]['reason'], "Authorization required")

  def testGetAclFeed(self):
    """Get an ACL feed"""
    self.cal_client.ProgrammaticLogin()
    feed = self.cal_client.GetCalendarAclFeed(self._aclFeedUri)
    self.assertNotEqual(0,len(feed.entry))

  def testGetAclEntryUnauthenticated(self):
    """Fiendishly try to get an ACL entry without authenticating"""
    try:
      self.cal_client.GetCalendarAclEntry(self._aclEntryUri)
      self.fail("Unauthenticated request should fail");
    except gdata.service.RequestError, error:
      self.assertEqual(error[0]['status'], 401)
      self.assertEqual(error[0]['reason'], "Authorization required")
  
  def testGetAclEntry(self):
    """Get an ACL entry"""
    self.cal_client.ProgrammaticLogin()
    self.cal_client.GetCalendarAclEntry(self._aclEntryUri)
    
  def testCalendarAclFeedFromString(self):
    """Create an ACL feed from a hard-coded string"""
    aclFeed = gdata.calendar.CalendarAclFeedFromString(test_data.ACL_FEED)
    self.assertEqual("Elizabeth Bennet's access control list", aclFeed.title.text)
    self.assertEqual(2,len(aclFeed.entry))
    
  def testCalendarAclEntryFromString(self):
    """Create an ACL entry from a hard-coded string"""
    aclEntry = gdata.calendar.CalendarAclEntryFromString(test_data.ACL_ENTRY)
    self.assertEqual("owner", aclEntry.title.text)
    self.assertEqual("user", aclEntry.scope.type)
    self.assertEqual("liz@gmail.com", aclEntry.scope.value)
    self.assertEqual("http://schemas.google.com/gCal/2005#owner", aclEntry.role.value)

  def testCreateAndDeleteAclEntry(self):
    """Add an ACL rule and verify that is it returned in the ACL feed.  Then delete the rule and 
       verify that the rule is no longer included in the ACL feed."""
    # Get the current number of ACL rules   
    self.cal_client.ProgrammaticLogin()
    aclFeed = self.cal_client.GetCalendarAclFeed(self._aclFeedUri)
    original_rule_count = len(aclFeed.entry)
 
    # Insert entry 
    rule = self._generateAclEntry()
    returned_rule = self.cal_client.InsertAclEntry(rule, self._aclFeedUri)

    # Verify rule was added with correct ACL values
    aclFeed = self.cal_client.GetCalendarAclFeed(self._aclFeedUri)
    self.assertEqual(original_rule_count+1, len(aclFeed.entry))
    self.assertEqualAclEntry(rule, returned_rule)

    # Delete the event
    self.cal_client.DeleteAclEntry(returned_rule.GetEditLink().href)
    aclFeed = self.cal_client.GetCalendarAclFeed(self._aclFeedUri)
    self.assertEquals(original_rule_count, len(aclFeed.entry))

  def testUpdateAclChangeScopeValue(self):
    """Fiendishly try to insert a test ACL rule and attempt to change the scope value (i.e. username).
       Verify that an exception is thrown, then delete the test rule."""
    # Insert a user-scoped owner role ot random user
    aclEntry = self._generateAclEntry("owner","user");
    self.cal_client.ProgrammaticLogin()
    rule = self._generateAclEntry()
    returned_rule = self.cal_client.InsertAclEntry(rule, self._aclFeedUri)

    # Change the scope value (i.e. what user is the owner) and update the entry
    updated_rule = returned_rule
    updated_rule.scope.value = "user_%s@gmail.com" % (self._getRandomNumber())
    try:
      returned_rule = self.cal_client.UpdateAclEntry(returned_rule.GetEditLink().href, updated_rule)
    except gdata.service.RequestError, error:
      self.assertEqual(error[0]['status'], 403)
      self.assertEqual(error[0]['reason'], "Forbidden")

    self.cal_client.DeleteAclEntry(updated_rule.GetEditLink().href)

  
  def testUpdateAclChangeScopeType(self):
    """Fiendishly try to insert a test ACL rule and attempt to change the scope type (i.e. from 'user' to 'domain').
       Verify that an exception is thrown, then delete the test rule."""
    # Insert a user-scoped owner role ot random user
    aclEntry = self._generateAclEntry("owner","user");
    self.cal_client.ProgrammaticLogin()
    rule = self._generateAclEntry()
    returned_rule = self.cal_client.InsertAclEntry(rule, self._aclFeedUri)

    # Change the scope value (i.e. what user is the owner) and update the entry
    updated_rule = returned_rule
    updated_rule.scope.type = "domain"
    try:
      returned_rule = self.cal_client.UpdateAclEntry(returned_rule.GetEditLink().href, updated_rule)
    except gdata.service.RequestError, error:
      self.assertEqual(error[0]['status'], 403)
      self.assertEqual(error[0]['reason'], "Forbidden")

    self.cal_client.DeleteAclEntry(updated_rule.GetEditLink().href)

    
  def testUpdateAclChangeRoleValue(self):
    """Insert a test ACL rule and attempt to change the scope type (i.e. from 'owner' to 'editor').
       Verify that an exception is thrown, then delete the test rule."""
    # Insert a user-scoped owner role ot random user
    aclEntry = self._generateAclEntry("owner","user");
    self.cal_client.ProgrammaticLogin()
    rule = self._generateAclEntry()
    returned_rule = self.cal_client.InsertAclEntry(rule, self._aclFeedUri)

    # Change the scope value (i.e. what user is the owner) and update the entry
    updated_rule = returned_rule
    updated_rule.role.value = "http://schemas.google.com/gCal/2005#editor"
    returned_rule = self.cal_client.UpdateAclEntry(returned_rule.GetEditLink().href, updated_rule)
    self.assertEqualAclEntry(updated_rule, returned_rule)
    
    self.cal_client.DeleteAclEntry(updated_rule.GetEditLink().href)
    
if __name__ == '__main__':
  print ('NOTE: Please run these tests only with a test account. ' +
      'The tests may delete or update your data.')
  username = raw_input('Please enter your username: ')
  password = getpass.getpass()
  unittest.main()




#!/usr/bin/python
#
# Copyright (C) 2007 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


"""Unit tests to exercise server interactions for blogger."""

__author__ = 'api.jscudder (Jeffrey Scudder)'

import unittest
import getpass
import atom
from gdata import test_data
import gdata.blogger
import gdata.blogger.service

username = ''
password = ''
test_blog_id = ''

class BloggerCrudTests(unittest.TestCase):
  
  def setUp(self):
    self.client = gdata.blogger.service.BloggerService(email=username, 
        password=password, source='GoogleInc-PythonBloggerUnitTests-1')
    # TODO: if the test_blog_id is not set, get the list of the user's blogs
    # and prompt for which blog to add the test posts to.
    self.client.ProgrammaticLogin()

  def testPostDraftUpdateAndDelete(self):
    new_entry = gdata.blogger.BlogPostEntry(title=atom.Title(
        text='Unit Test Post'))
    new_entry.content = atom.Content('text', None, 'Hello World')
    # Make this post a draft so it will not appear publicly on the blog.
    new_entry.control = atom.Control(draft=atom.Draft(text='yes'))
    new_entry.AddLabel('test')

    posted = self.client.AddPost(new_entry, blog_id=test_blog_id)

    self.assertEquals(posted.title.text, new_entry.title.text)
    # Should be one category in the posted entry for the 'test' label.
    self.assertEquals(len(posted.category), 1)
    self.assert_(isinstance(posted, gdata.blogger.BlogPostEntry))

    # Change the title and add more labels.
    posted.title.text = 'Updated'
    posted.AddLabel('second')
    updated = self.client.UpdatePost(entry=posted)

    self.assertEquals(updated.title.text, 'Updated')
    self.assertEquals(len(updated.category), 2)

    # Cleanup and delete the draft blog post.
    self.client.DeletePost(entry=posted)

  def testAddComment(self):
    # Create a test post to add comments to.
    new_entry = gdata.blogger.BlogPostEntry(title=atom.Title(
            text='Comments Test Post'))
    new_entry.content = atom.Content('text', None, 'Hello Comments')
    target_post = self.client.AddPost(new_entry, blog_id=test_blog_id)

    blog_id = target_post.GetBlogId()
    post_id = target_post.GetPostId()

    new_comment = gdata.blogger.CommentEntry()
    new_comment.content = atom.Content(text='Test comment')
    posted = self.client.AddComment(new_comment, blog_id=blog_id, 
        post_id=post_id)
    self.assertEquals(posted.content.text, new_comment.content.text)

    # Cleanup and delete the comment test blog post.
    self.client.DeletePost(entry=target_post)


class BloggerQueryTests(unittest.TestCase):

  def testConstructBlogQuery(self):
    pass

  def testConstructBlogQuery(self):
    pass

  def testConstructBlogQuery(self):
    pass


if __name__ == '__main__':
  print ('NOTE: Please run these tests only with a test account. ' +
         'The tests may delete or update your data.')
  username = raw_input('Please enter your username: ')
  password = getpass.getpass()
  test_blog_id = raw_input('Please enter the blog id for the test blog: ')
  unittest.main()




#!/usr/bin/env python
#
# Copyright 2009 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Contains a Sample for Google Profiles.

  ProfilesSample: demonstrates operations with the Profiles feed.
"""

__author__ = 'jtoledo (Julian Toledo)'


import getopt
import getpass
import sys

import gdata.contacts
import gdata.contacts.service


class ProfilesSample(object):
  """ProfilesSample object demonstrates operations with the Profiles feed."""

  def __init__(self, email, password, domain):
    """Constructor for the ProfilesSample object.

    Takes an email and password corresponding to a gmail account to
    demonstrate the functionality of the Profiles feed.

    Args:
      email: [string] The e-mail address of the account to use for the sample.
      password: [string] The password corresponding to the account specified by
          the email parameter.
      domain: [string] The domain for the Profiles feed
    """
    self.gd_client = gdata.contacts.service.ContactsService(
        contact_list=domain)
    self.gd_client.email = email
    self.gd_client.password = password
    self.gd_client.source = 'GoogleInc-ProfilesPythonSample-1'
    self.gd_client.ProgrammaticLogin()

  def PrintFeed(self, feed, ctr=0):
    """Prints out the contents of a feed to the console.

    Args:
      feed: A gdata.profiles.ProfilesFeed instance.
      ctr: [int] The number of entries in this feed previously printed. This
          allows continuous entry numbers when paging through a feed.

    Returns:
      The number of entries printed, including those previously printed as
      specified in ctr. This is for passing as an ar1gument to ctr on
      successive calls to this method.
    """
    if not feed.entry:
      print '\nNo entries in feed.\n'
      return 0
    for entry in feed.entry:
      self.PrintEntry(entry)
    return len(feed.entry) + ctr

  def PrintEntry(self, entry):
    """Prints out the contents of a single Entry to the console.

    Args:
      entry: A gdata.contacts.ProfilesEntry
    """
    print '\n%s' % (entry.title.text)    
    for email in entry.email:
      if email.primary == 'true':
        print 'Email: %s (primary)' % (email.address)
      else:
        print 'Email: %s' % (email.address)
    if entry.nickname:
      print 'Nickname: %s' % (entry.nickname.text)
    if entry.occupation:
      print 'Occupation: %s' % (entry.occupation.text)
    if entry.gender:
      print 'Gender: %s' % (entry.gender.value)
    if entry.birthday:
      print 'Birthday: %s' % (entry.birthday.when)
    for relation in entry.relation:
      print 'Relation: %s %s' % (relation.rel, relation.text)
    for user_defined_field in entry.user_defined_field:
      print 'UserDefinedField: %s %s' % (user_defined_field.key,
                                         user_defined_field.value)
    for website in entry.website:
      print 'Website: %s %s' % (website.href, website.rel)
    for phone_number in entry.phone_number:
      print 'Phone Number: %s' % phone_number.text
    for organization in entry.organization:
      print 'Organization:'
      if organization.org_name:
        print ' Name: %s' % (organization.org_name.text)
      if organization.org_title:
        print ' Title: %s' % (organization.org_title.text)
      if organization.org_department:
        print ' Department: %s' % (organization.org_department.text)
      if organization.org_job_description:
        print ' Job Desc: %s' % (organization.org_job_description.text)

  def PrintPaginatedFeed(self, feed, print_method):
    """Print all pages of a paginated feed.

    This will iterate through a paginated feed, requesting each page and
    printing the entries contained therein.

    Args:
      feed: A gdata.contacts.ProfilesFeed instance.
      print_method: The method which will be used to print each page of the
    """
    ctr = 0
    while feed:
      # Print contents of current feed
      ctr = print_method(feed=feed, ctr=ctr)
      # Prepare for next feed iteration
      next = feed.GetNextLink()
      feed = None
      if next:
        if self.PromptOperationShouldContinue():
          # Another feed is available, and the user has given us permission
          # to fetch it
          feed = self.gd_client.GetProfilesFeed(next.href)
        else:
          # User has asked us to terminate
          feed = None

  def PromptOperationShouldContinue(self):
    """Display a "Continue" prompt.

    This give is used to give users a chance to break out of a loop, just in
    case they have too many profiles/groups.

    Returns:
      A boolean value, True if the current operation should continue, False if
      the current operation should terminate.
    """
    while True:
      key_input = raw_input('Continue [Y/n]? ')
      if key_input is 'N' or key_input is 'n':
        return False
      elif key_input is 'Y' or key_input is 'y' or key_input is '':
        return True

  def ListAllProfiles(self):
    """Retrieves a list of profiles and displays name and primary email."""
    feed = self.gd_client.GetProfilesFeed()
    self.PrintPaginatedFeed(feed, self.PrintFeed)

  def SelectProfile(self):
    username = raw_input('Please enter your username for the profile: ')
    entry_uri = self.gd_client.GetFeedUri('profiles')+'/'+username
    try:
      entry = self.gd_client.GetProfile(entry_uri)
      self.PrintEntry(entry)
    except gdata.service.RequestError:
      print 'Invalid username for the profile.'

  def PrintMenu(self):
    """Displays a menu of options for the user to choose from."""
    print ('\nProfiles Sample\n'
           '1) List all of your Profiles.\n'
           '2) Get a single Profile.\n'
           '3) Exit.\n')

  def GetMenuChoice(self, maximum):
    """Retrieves the menu selection from the user.

    Args:
      maximum: [int] The maximum number of allowed choices (inclusive)

    Returns:
      The integer of the menu item chosen by the user.
    """
    while True:
      key_input = raw_input('> ')

      try:
        num = int(key_input)
      except ValueError:
        print 'Invalid choice. Please choose a value between 1 and', maximum
        continue

      if num > maximum or num < 1:
        print 'Invalid choice. Please choose a value between 1 and', maximum
      else:
        return num

  def Run(self):
    """Prompts the user to choose funtionality to be demonstrated."""
    try:
      while True:
        self.PrintMenu()
        choice = self.GetMenuChoice(3)
        if choice == 1:
          self.ListAllProfiles()
        elif choice == 2:
          self.SelectProfile()
        elif choice == 3:
          return

    except KeyboardInterrupt:
      print '\nGoodbye.'
      return


def main():
  """Demonstrates use of the Profiles using the ProfilesSample object."""
  # Parse command line options
  try:
    opts, args = getopt.getopt(sys.argv[1:], '', ['user=', 'pw=', 'domain='])
  except getopt.error, msg:
    print 'python profiles_example.py --user [username] --pw [password]'
    print ' --domain [domain]'
    sys.exit(2)

  user = ''
  pw = ''
  domain = ''

  # Process options
  for option, arg in opts:
    if option == '--user':
      user = arg
    elif option == '--pw':
      pw = arg
    elif option == '--domain':
      domain = arg

  while not user:
    print 'NOTE: Please run these tests only with a test account.'
    user = raw_input('Please enter your email: ')
  while not pw:
    pw = getpass.getpass('Please enter password: ')
    if not pw:
      print 'Password cannot be blank.'
  while not domain:
    domain = raw_input('Please enter your Apps domain: ')

  try:
    sample = ProfilesSample(user, pw, domain)
  except gdata.service.BadAuthentication:
    print 'Invalid user credentials given.'
    return

  sample.Run()

if __name__ == '__main__':
  main()



#!/usr/bin/env python
#
#    Copyright (C) 2009 Google Inc.
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.


# This module is used for version 2 of the Google Data APIs.


__author__ = 'j.s@google.com (Jeff Scudder)'


import unittest
import all_tests
import gdata.test_config as conf


conf.options.set_value('runlive', 'true')
conf.options.set_value('savecache', 'true')
conf.options.set_value('clearcache', 'false')


def suite():
  return unittest.TestSuite((atom_tests.core_test.suite(),))

if __name__ == '__main__':
  unittest.TextTestRunner().run(all_tests.suite())



#!/usr/bin/env python
#
# Copyright (C) 2010 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Functional Tests for Google Analytics Account Feed and Data Feed.

AnalyticsClientTest: Tests making live requests to Google Analytics API.
"""

__author__ = 'api.nickm@google.com (Nick Mihailovski)'


import unittest
import gdata.client
import gdata.data
import gdata.gauth
import gdata.analytics.client
import gdata.test_config as conf


conf.options.register_option(conf.GA_TABLE_ID)


class AnalyticsClientTest(unittest.TestCase):
  """Tests creating an Account Feed query and making a request to the
  Google Analytics Account Feed."""

  def setUp(self):
    """Creates an AnalyticsClient object."""

    self.client = None
    if conf.options.get_value('runlive') == 'true':
      self.client = gdata.analytics.client.AnalyticsClient()
      self.client.http_client.debug = True

      conf.configure_client(
          self.client,
          'AnalyticsClientTest',
          self.client.auth_service)

  def testAccountFeed(self):
    """Tests if the Account Feed exists."""

    if not conf.options.get_value('runlive') == 'true':
      return
    conf.configure_cache(self.client, 'testAccountFeed')

    account_query = gdata.analytics.client.AccountFeedQuery({
        'max-results': '1'
    })

    feed = self.client.GetAccountFeed(account_query)
    self.assert_(feed.entry is not None)

    properties = [
        'ga:accountId',
        'ga:accountName',
        'ga:profileId',
        'ga:webPropertyId',
        'ga:currency',
        'ga:timezone'
    ]

    entry = feed.entry[0]
    for prop in properties:
      property = entry.GetProperty(prop)
      self.assertEquals(property.name, prop)

  def testDataFeed(self):
    """Tests if the Data Feed exists."""

    start_date = '2008-10-01'
    end_date = '2008-10-02'
    metrics = 'ga:visits'

    if not conf.options.get_value('runlive') == 'true':
      return
    conf.configure_cache(self.client, 'testDataFeed')

    data_query = gdata.analytics.client.DataFeedQuery({
      'ids': conf.options.get_value('table_id'),
      'start-date': start_date,
      'end-date': end_date,
      'metrics' : metrics,
      'max-results': '1'
    })
    feed = self.client.GetDataFeed(data_query)

    self.assert_(feed.entry is not None)
    self.assertEquals(feed.start_date.text, start_date)
    self.assertEquals(feed.end_date.text, end_date)
    self.assertEquals(feed.entry[0].GetMetric(metrics).name, metrics)

  def testManagementFeed(self):
    """Tests of the Management Feed exists."""

    if not conf.options.get_value('runlive') == 'true':
      return
    conf.configure_cache(self.client, 'testManagementFeed')

    account_query = gdata.analytics.client.AccountQuery()
    feed = self.client.GetManagementFeed(account_query)

    self.assert_(feed.entry is not None)

  def tearDown(self):
    """Closes client connection."""
    conf.close_client(self.client)


def suite():
  return conf.build_suite([AnalyticsClientTest])


if __name__ == '__main__':
  unittest.TextTestRunner().run(suite())




#!/usr/bin/env python
#
# Copyright (C) 2009 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


# This module is used for version 2 of the Google Data APIs.
# These tests attempt to connect to Google servers.


__author__ = 'j.s@google.com (Jeff Scudder)'


import unittest
import gdata.test_config as conf
import gdata.contacts.client
import atom.core
import atom.data
import gdata.data


class ContactsTest(unittest.TestCase):

  def setUp(self):
    self.client = None
    if conf.options.get_value('runlive') == 'true':
      self.client = gdata.contacts.client.ContactsClient()
      conf.configure_client(self.client, 'ContactsTest', 'cp')

  def tearDown(self):
    conf.close_client(self.client)

  def test_create_update_delete_contact(self):
    if not conf.options.get_value('runlive') == 'true':
      return
    # Either load the recording or prepare to make a live request.
    conf.configure_cache(self.client, 'test_create_update_delete_contact')

    new_contact = gdata.contacts.data.ContactEntry(
        nickname=gdata.contacts.data.NickName(text='Joe'),
        name=gdata.data.Name(
            given_name=gdata.data.GivenName(text='Joseph'),
            family_name=gdata.data.FamilyName(text='Testerson')))
    new_contact.birthday = gdata.contacts.data.Birthday(when='2009-11-11')
    new_contact.language.append(gdata.contacts.data.Language(
        label='German'))
    created = self.client.create_contact(new_contact)

    # Add another language.
    created.language.append(gdata.contacts.data.Language(
        label='French'))

    # Create a new membership group for our test contact.
    new_group = gdata.contacts.data.GroupEntry(
        title=atom.data.Title(text='a test group'))
    created_group = self.client.create_group(new_group)

    self.assert_(created_group.id.text)

    # Add the contact to the new group.
    created.group_membership_info.append(
        gdata.contacts.data.GroupMembershipInfo(href=created_group.id.text))

    # Upload the changes to the language and group membership.
    edited = self.client.update(created)

    # Delete the group and the test contact.
    self.client.delete(created_group)
    self.client.delete(edited)

  def test_low_level_create_update_delete(self):
    if not conf.options.get_value('runlive') == 'true':
      return
    # Either load the recording or prepare to make a live request.
    conf.configure_cache(self.client, 'test_low_level_create_update_delete')
    entry = atom.data.Entry()
    entry.title = atom.data.Title(text='Jeff')
    entry._other_elements.append(
        gdata.data.Email(rel=gdata.data.WORK_REL, address='j.s@google.com'))

    http_request = atom.http_core.HttpRequest()
    http_request.add_body_part(entry.to_string(), 'application/atom+xml')
    posted = self.client.request('POST', 
        'http://www.google.com/m8/feeds/contacts/default/full',
        desired_class=atom.data.Entry, http_request=http_request)

    self_link = None
    edit_link = None
    for link in posted.get_elements('link', 'http://www.w3.org/2005/Atom'):
      if link.get_attributes('rel')[0].value == 'self':
        self_link = link.get_attributes('href')[0].value
      elif link.get_attributes('rel')[0].value == 'edit':
        edit_link = link.get_attributes('href')[0].value
    self.assert_(self_link is not None)
    self.assert_(edit_link is not None)

    etag = posted.get_attributes('etag')[0].value
    self.assert_(etag is not None)
    self.assert_(len(etag) > 0)

    # Delete the test contact.
    http_request = atom.http_core.HttpRequest()
    http_request.headers['If-Match'] = etag
    self.client.request('DELETE', edit_link, http_request=http_request)

def suite():
  return conf.build_suite([ContactsTest])


if __name__ == '__main__':
  unittest.TextTestRunner().run(suite())



#!/usr/bin/python2.4
#
# Copyright 2012 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Sample to demonstrate the Email Audit API's email monitoring functions.

The sample demonstrates the creating, updating, retrieving and deleting of
email monitors.
"""

__author__ = 'Gunjan Sharma <gunjansharma@google.com>'

from datetime import datetime
import getopt
import re
import sys
import gdata
from gdata.apps.audit.service import AuditService


class EmailMonitoringException(Exception):
  """Exception class for EmailMonitoring, shows appropriate error message."""


class EmailMonitoring(object):
  """Sample demonstrating how to perform CRUD operations on email monitor."""

  def __init__(self, consumer_key, consumer_secret, domain):
    """Create a new EmailMonitoring object configured for a domain.

    Args:
      consumer_key: A string representing a consumerKey.
      consumer_secret: A string representing a consumerSecret.
      domain: A string representing the domain to work on in the sample.
    """
    self.consumer_key = consumer_key
    self.consumer_secret = consumer_secret
    self.domain = domain
    self._Authorize()

  def _Authorize(self):
    """Asks the domain's admin to authorize access to the apps Apis."""
    self.service = AuditService(domain=self.domain, source='emailAuditSample')
    self.service.SetOAuthInputParameters(
        gdata.auth.OAuthSignatureMethod.HMAC_SHA1,
        self.consumer_key, self.consumer_secret)
    request_token = self.service.FetchOAuthRequestToken()
    self.service.SetOAuthToken(request_token)
    auth_url = self.service.GenerateOAuthAuthorizationURL()
    print auth_url
    raw_input('Manually go to the above URL and authenticate.'
              'Press Return after authorization.')
    self.service.UpgradeToOAuthAccessToken()

  def _CheckUsername(self, username):
    """Checks if a given username is valid or not.

    Args:
      username: A string to check for validity.

    Returns:
      True if username is valid, False otherwise.
    """
    if len(username) > 64:
      print 'Username length should be less than 64'
      return False
    pattern = re.compile('[^\w\.\+-_\']+')
    return not bool(pattern.search(username))

  def _GetValidUsername(self, typeof):
    """Takes a valid username as input.

    Args:
      typeof: A string representing the type of user.

    Returns:
      A valid string corresponding to username.
    """
    username = ''
    while not username:
      username = raw_input('Enter a valid %s username: ' % typeof)
      if not self._CheckUsername(username):
        print 'Invalid username'
        username = ''
    return username

  def _GetValidDate(self, is_neccessary):
    """Takes a valid date as input in 'yyyy-mm-dd HH:MM' format.

    Args:
      is_neccessary: A boolean denoting if a non empty value is needed.

    Returns:
      A valid string corresponding to date.
    """
    date = ''
    extra_stmt = ''
    if not is_neccessary:
      extra_stmt = '. Press enter to skip.'
    while not date:
      date = raw_input(
          'Enter a valid date as (yyyy-mm-dd HH:MM)%s:' % extra_stmt)
      if not (date and is_neccessary):
        return date
      try:
        datetime.strptime(date, '%Y-%m-%d %H:%M')
        return date
      except ValueError:
        print 'Not a valid date!'
        date = ''

  def _GetBool(self, name):
    """Takes a boolean value as input.

    Args:
      name: A string for which input is to be taken.

    Returns:
      A boolean for an entity represented by name.
    """
    choice = raw_input(
        'Enter your choice (t/f) for %s (defaults to False):' % name).strip()
    if choice == 't':
      return True
    return False

  def _CreateEmailMonitor(self):
    """Creates/Updates an email monitor."""
    src_user = self._GetValidUsername('source')
    dest_user = self._GetValidUsername('destination')
    end_date = self._GetValidDate(True)
    start_date = self._GetValidDate(False)
    incoming_headers = self._GetBool('incoming headers')
    outgoing_headers = self._GetBool('outgoing headers')
    drafts = self._GetBool('drafts')
    drafts_headers = False
    if drafts:
      drafts_headers = self._GetBool('drafts headers')
    chats = self._GetBool('chats')
    chats_headers = False
    if chats:
      self._GetBool('chats headers')
    self.service.createEmailMonitor(
        src_user, dest_user,
        end_date, start_date,
        incoming_headers, outgoing_headers,
        drafts, drafts_headers,
        chats, chats_headers)
    print 'Email monitor created/updated successfully!\n'

  def _RetrieveEmailMonitor(self):
    """Retrieves all email monitors for a user."""
    src_user = self._GetValidUsername('source')
    monitors = self.service.getEmailMonitors(src_user)
    for monitor in monitors:
      for key in monitor.keys():
        print '%s ----------- %s' % (key, monitor.get(key))
      print ''
    print 'Email monitors retrieved successfully!\n'

  def _DeleteEmailMonitor(self):
    """Deletes an email monitor."""
    src_user = self._GetValidUsername('source')
    dest_user = self._GetValidUsername('destination')
    self.service.deleteEmailMonitor(src_user, dest_user)
    print 'Email monitor deleted successfully!\n'

  def Run(self):
    """Handles the flow of the sample."""
    functions_list = [
        {
            'function': self._CreateEmailMonitor,
            'description': 'Create a email monitor for a domain user'
        },
        {
            'function': self._CreateEmailMonitor,
            'description': 'Update a email monitor for a domain user'
        },
        {
            'function': self._RetrieveEmailMonitor,
            'description': 'Retrieve all email monitors for a domain user'
        },
        {
            'function': self._DeleteEmailMonitor,
            'description': 'Delete a email monitor for a domain user'
        }
    ]

    while True:
      print 'What would you like to do? Choose an option:'
      print '0 - To exit'
      for i in range (0, len(functions_list)):
        print '%d - %s' % ((i + 1), functions_list[i].get('description'))
      choice = raw_input('Enter your choice: ').strip()
      if choice.isdigit():
        choice = int(choice)
      if choice == 0:
        break
      if choice < 0 or choice > len(functions_list):
        print 'Not a valid option!'
        continue
      try:
        functions_list[choice - 1].get('function')()
      except gdata.apps.service.AppsForYourDomainException, e:
        if e.error_code == 1301:
          print '\nError: Invalid username!!\n'
        else:
          raise e


def PrintUsageString():
  """Prints the correct call for running the sample."""
  print ('python email_audit_email_monitoring.py '
         '--consumer_key [ConsumerKey] --consumer_secret [ConsumerSecret] '
         '--domain [domain]')


def main():
  """Runs the sample using an instance of EmailMonitoring."""

  try:
    opts, args = getopt.getopt(sys.argv[1:], '', ['consumer_key=',
                                                  'consumer_secret=',
                                                  'domain='])
  except getopt.error, msg:
    PrintUsageString()
    sys.exit(1)

  consumer_key = ''
  consumer_secret = ''
  domain = ''
  for option, arg in opts:
    if option == '--consumer_key':
      consumer_key = arg
    elif option == '--consumer_secret':
      consumer_secret = arg
    elif option == '--domain':
      domain = arg

  if not (consumer_key and consumer_secret and domain):
    print 'Requires exactly three flags.'
    PrintUsageString()
    sys.exit(1)

  try:
    email_monitoring = EmailMonitoring(
        consumer_key, consumer_secret, domain)
    email_monitoring.Run()
  except gdata.apps.service.AppsForYourDomainException, e:
    raise EmailMonitoringException('Invalid Domain')
  except gdata.service.FetchingOAuthRequestTokenFailed, e:
    raise EmailMonitoringException('Invalid consumer credentials')
  except Exception, e:
    if e.args[0].get('status') == 503:
      raise EmailMonitoringException('Server busy')
    elif e.args[0].get('status') == 500:
      raise EmailMonitoringException('Internal server error')
    else:
      raise EmailMonitoringException('Unknown error')


if __name__ == '__main__':
  main()



#!/usr/bin/env python
#
# Copyright (C) 2009 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


# This module is used for version 2 of the Google Data APIs.
# These tests attempt to connect to Google servers.


__author__ = 'j.s@google.com (Jeff Scudder)'


import unittest
import gdata.spreadsheets.client
import gdata.gauth
import gdata.client
import atom.http_core
import atom.mock_http_core
import atom.core
import gdata.data
import gdata.test_config as conf


conf.options.register_option(conf.SPREADSHEET_ID_OPTION)


class SpreadsheetsClientTest(unittest.TestCase):

  def setUp(self):
    self.client = None
    if conf.options.get_value('runlive') == 'true':
      self.client = gdata.spreadsheets.client.SpreadsheetsClient()
      conf.configure_client(self.client, 'SpreadsheetsClientTest', 'wise')

  def tearDown(self):
    conf.close_client(self.client)

  def test_create_update_delete_worksheet(self):
    if not conf.options.get_value('runlive') == 'true':
      return
    # Either load the recording or prepare to make a live request.
    conf.configure_cache(self.client, 'test_create_update_delete_worksheet')

    spreadsheet_id = conf.options.get_value('spreadsheetid')
    original_worksheets = self.client.get_worksheets(spreadsheet_id)
    self.assert_(isinstance(original_worksheets,
                               gdata.spreadsheets.data.WorksheetsFeed))
    worksheet_count = int(original_worksheets.total_results.text)

    # Add a new worksheet to the spreadsheet.
    created = self.client.add_worksheet(
        spreadsheet_id, 'a test worksheet', 4, 8)
    self.assert_(isinstance(created,
                               gdata.spreadsheets.data.WorksheetEntry))
    self.assertEqual(created.title.text, 'a test worksheet')
    self.assertEqual(created.row_count.text, '4')
    self.assertEqual(created.col_count.text, '8')

    # There should now be one more worksheet in this spreadsheet.
    updated_worksheets = self.client.get_worksheets(spreadsheet_id)
    new_worksheet_count = int(updated_worksheets.total_results.text)
    self.assertEqual(worksheet_count + 1, new_worksheet_count)

    # Delete our test worksheet.
    self.client.delete(created)
    # We should be back to the original number of worksheets.
    updated_worksheets = self.client.get_worksheets(spreadsheet_id)
    new_worksheet_count = int(updated_worksheets.total_results.text)
    self.assertEqual(worksheet_count, new_worksheet_count)

  def test_create_update_delete_table_and_records(self):
    if not conf.options.get_value('runlive') == 'true':
      return
    # Either load the recording or prepare to make a live request.
    conf.configure_cache(
        self.client, 'test_create_update_delete_table_and_records')

    spreadsheet_id = conf.options.get_value('spreadsheetid')
    tables = self.client.get_tables(spreadsheet_id)

    test_worksheet = self.client.add_worksheet(
        spreadsheet_id, 'worksheet x', rows=30, cols=3)

    self.assert_(isinstance(tables, gdata.spreadsheets.data.TablesFeed))
    initial_count = tables.total_results.text

    created_table = self.client.add_table(
        spreadsheet_id, 'Test Table', 'This table is for testing',
        'worksheet x', header_row=5, num_rows=10, start_row=8,
        insertion_mode=None,
        column_headers={'B': 'Food', 'C': 'Drink', 'A': 'Price'})

    # Re-get the list of tables and make sure there are more now.
    updated_tables = self.client.get_tables(spreadsheet_id)
    self.assertEqual(int(initial_count) + 1, 
                     int(updated_tables.total_results.text))

    # Get the records in our new table to make sure it has the correct
    # number of records.
    table_num = int(created_table.get_table_id())
    starting_records = self.client.get_records(spreadsheet_id, table_num)
    self.assertEqual(starting_records.total_results.text, '10')
    self.assert_(starting_records.entry[0].field[0].text is None)
    self.assert_(starting_records.entry[0].field[1].text is None)
    self.assert_(starting_records.entry[1].field[0].text is None)
    self.assert_(starting_records.entry[1].field[1].text is None)

    record1 = self.client.add_record(
        spreadsheet_id, table_num,
        {'Food': 'Cheese', 'Drink': 'Soda', 'Price': '2.99'}, 'icky')
    self.client.add_record(spreadsheet_id, table_num,
                           {'Food': 'Eggs', 'Drink': 'Milk'})
    self.client.add_record(spreadsheet_id, table_num,
                           {'Food': 'Spinach', 'Drink': 'Water'})

    updated_records = self.client.get_records(spreadsheet_id, table_num)
    self.assertEqual(updated_records.entry[10].value_for_name('Price'), '2.99')
    self.assertEqual(updated_records.entry[10].value_for_index('A'), '2.99')
    self.assertEqual(updated_records.entry[10].value_for_name('Drink'),
                     'Soda')
    self.assert_(updated_records.entry[11].value_for_name('Price') is None)
    self.assertEqual(updated_records.entry[11].value_for_name('Drink'),
                     'Milk')
    self.assertEqual(updated_records.entry[12].value_for_name('Drink'),
                     'Water')
    self.assert_(updated_records.entry[1].value_for_index('A') is None)
    self.assert_(updated_records.entry[2].value_for_index('B') is None)
    self.assert_(updated_records.entry[3].value_for_index('C') is None)

    # Cleanup the table.
    self.client.delete(created_table)
    # Delete the test worksheet in which the table was placed.
    self.client.delete(test_worksheet)

    # Make sure we are back to the original count.
    updated_tables = self.client.get_tables(spreadsheet_id)
    self.assertEqual(int(initial_count), 
                     int(updated_tables.total_results.text))

  def test_get_and_update_cell(self):
    if not conf.options.get_value('runlive') == 'true':
      return
    # Either load the recording or prepare to make a live request.
    conf.configure_cache(self.client, 'test_get_and_update_cell')

    spreadsheet_id = conf.options.get_value('spreadsheetid')

    test_worksheet = self.client.add_worksheet(
        spreadsheet_id, 'worksheet x', rows=30, cols=3)

    # Get a cell and set its value.
    cell_entry = self.client.get_cell(
        spreadsheet_id, test_worksheet.get_worksheet_id(), 1, 1)
    cell_entry.cell.input_value = 'a test'
    result = self.client.update(cell_entry)
    self.assertEquals(cell_entry.cell.input_value, result.cell.input_value)

    # Verify that the value was set.
    cells = self.client.get_cells(
        spreadsheet_id, test_worksheet.get_worksheet_id())
    self.assertEquals(len(cells.entry), 1)
    self.assertEquals(cells.entry[0].cell.input_value, 'a test')

    # Delete the test worksheet.
    self.client.delete(test_worksheet, force=True)

  def set_cell(self, spreadsheet_id, worksheet_id, row, column, value):
    cell_entry = self.client.get_cell(
        spreadsheet_id, worksheet_id, row, column)
    self.assert_(cell_entry is not None)
    cell_entry.cell.input_value = value
    self.assert_(self.client.update(cell_entry) is not None)

  def test_batch_set_cells(self):
    if not conf.options.get_value('runlive') == 'true':
      return
    # Either load the recording or prepare to make a live request.
    conf.configure_cache(self.client, 'test_get_and_update_cell')

    spreadsheet_id = conf.options.get_value('spreadsheetid')

    test_worksheet = self.client.add_worksheet(
        spreadsheet_id, 'worksheet x', rows=30, cols=3)

    # Set a couple of cells in a batch request.
    feed = gdata.spreadsheets.data.build_batch_cells_update(
        spreadsheet_id, test_worksheet.get_worksheet_id())
    feed.add_set_cell(1, 1, '5')
    feed.add_set_cell(1, 2, '=A1+2')
    result = self.client.batch(feed, force=True)
    self.assertEqual(result.entry[0].cell.text, '5')
    self.assertEqual(result.entry[1].cell.text, '7')

    # Delete the test worksheet.
    self.client.delete(test_worksheet, force=True)

  def test_crud_on_list_feed(self):
    if not conf.options.get_value('runlive') == 'true':
      return
    # Either load the recording or prepare to make a live request.
    conf.configure_cache(self.client, 'test_crud_on_list_feed')

    spreadsheet_id = conf.options.get_value('spreadsheetid')

    test_worksheet = self.client.add_worksheet(
        spreadsheet_id, 'worksheet x', rows=30, cols=3)
    worksheet_id = test_worksheet.get_worksheet_id()
    
    # Create the first column to provide row headings.
    self.set_cell(spreadsheet_id, worksheet_id, 1, 1, 'cola')
    self.set_cell(spreadsheet_id, worksheet_id, 1, 2, 'colb')
    self.set_cell(spreadsheet_id, worksheet_id, 1, 3, 'colc')

    # Add a row to the spreadsheet.
    entry = gdata.spreadsheets.data.ListEntry()
    entry.from_dict({'cola': 'alpha', 'colb': 'beta', 'colc': 'gamma'})
    added = self.client.add_list_entry(entry, spreadsheet_id, worksheet_id)
    self.assert_(isinstance(added, gdata.spreadsheets.data.ListEntry))
    self.assertEquals(added.get_value('cola'), 'alpha')

    # Update the row.
    added.from_dict({'cola': '1', 'colb': '2', 'colc': '3'})
    updated = self.client.update(added)
    self.assert_(isinstance(updated, gdata.spreadsheets.data.ListEntry))
    self.assertEquals(updated.get_value('cola'), '1')

    # Check the number of rows.
    rows = self.client.get_list_feed(spreadsheet_id, worksheet_id)
    self.assertEquals(len(rows.entry), 1)

    # Remove the row.
    self.client.delete(updated)

    # Check that it was removed.
    rows = self.client.get_list_feed(spreadsheet_id, worksheet_id)
    self.assertEquals(len(rows.entry), 0)

    # Delete the test worksheet.
    self.client.delete(test_worksheet, force=True)


def suite():
  return conf.build_suite([SpreadsheetsClientTest])


if __name__ == '__main__':
  unittest.TextTestRunner().run(suite())



# -*- coding: utf-8 -*-
"""
Copyright (c) 2008, appengine-utilities project
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
- Redistributions of source code must retain the above copyright notice, this
  list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice,
  this list of conditions and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
- Neither the name of the appengine-utilities project nor the names of its
  contributors may be used to endorse or promote products derived from this
  software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""

# main python imports
import os
import time
import datetime
import random
import hashlib
import Cookie
import pickle
import sys
import logging
from time import strftime

# google appengine imports
from google.appengine.ext import db
from google.appengine.api import memcache

from django.utils import simplejson

# settings
try:
    import settings_default
    import settings

    if settings.__name__.rsplit('.', 1)[0] != settings_default.__name__.rsplit('.', 1)[0]:
        settings = settings_default
except:
    settings = settings_default




class _AppEngineUtilities_Session(db.Model):
    """
    Model for the sessions in the datastore. This contains the identifier and
    validation information for the session.
    """

    sid = db.StringListProperty()
    ip = db.StringProperty()
    ua = db.StringProperty()
    last_activity = db.DateTimeProperty()
    dirty = db.BooleanProperty(default=False)
    working = db.BooleanProperty(default=False)
    deleted = db.BooleanProperty(default=False) 

    def put(self):
        """
        Extends put so that it writes vaules to memcache as well as the
        datastore, and keeps them in sync, even when datastore writes fails.

        Returns the session object.
        """
        try:
            memcache.set(u"_AppEngineUtilities_Session_%s" % \
                (str(self.key())), self)
        except:
            # new session, generate a new key, which will handle the
            # put and set the memcache
            db.put(self)

        self.last_activity = datetime.datetime.now()

        try:
            self.dirty = False
            db.put(self)
            memcache.set(u"_AppEngineUtilities_Session_%s" % \
                (str(self.key())), self)
        except:
            self.dirty = True
            memcache.set(u"_AppEngineUtilities_Session_%s" % \
                (str(self.key())), self)

        return self

    @classmethod
    def get_session(cls, session_obj=None):
        """
        Uses the passed objects sid to get a session object from memcache,
        or datastore if a valid one exists.

        Args:
            session_obj: a session object

        Returns a validated session object.
        """
        if session_obj.sid == None:
            return None
        session_key = session_obj.sid.split(u'_')[0]
        session = memcache.get(u"_AppEngineUtilities_Session_%s" % \
            (str(session_key)))
        if session:
            if session.deleted == True:
                session.delete()
                return None
            if session.dirty == True and session.working != False:
                # the working bit is used to make sure multiple requests,
                # which can happen with ajax oriented sites, don't try to put
                # at the same time
                session.working = True
                memcache.set(u"_AppEngineUtilities_Session_%s" % \
                    (str(session_key)), session)
                session.put()
            if session_obj.sid in session.sid:
                sessionAge = datetime.datetime.now() - session.last_activity
                if sessionAge.seconds > session_obj.session_expire_time:
                    session.delete()
                    return None
                return session
            else:
                return None
 
        # Not in memcache, check datastore
        
        ds_session = db.get(str(session_key))
        if ds_session:
          sessionAge = datetime.datetime.now() - ds_session.last_activity
          if sessionAge.seconds > session_obj.session_expire_time:
              ds_session.delete()
              return None
          memcache.set(u"_AppEngineUtilities_Session_%s" % \
              (str(session_key)), ds_session)
          memcache.set(u"_AppEngineUtilities_SessionData_%s" % \
              (str(session_key)), ds_session.get_items_ds())
        return ds_session


    def get_items(self):
        """
        Returns all the items stored in a session. Queries memcache first
        and will try the datastore next.
        """
        items = memcache.get(u"_AppEngineUtilities_SessionData_%s" % \
            (str(self.key())))
        if items:
            for item in items:
                if item.deleted == True:
                    item.delete()
                    items.remove(item)
            return items

        query = _AppEngineUtilities_SessionData.all()
        query.filter(u"session", self)
        results = query.fetch(1000)
        return results

    def get_item(self, keyname = None):
        """
        Returns a single session data item from the memcache or datastore

        Args:
            keyname: keyname of the session data object

        Returns the session data object if it exists, otherwise returns None
        """
        mc = memcache.get(u"_AppEngineUtilities_SessionData_%s" % \
            (str(self.key())))
        if mc:
            for item in mc:
                if item.keyname == keyname:
                    if item.deleted == True:
                        item.delete()
                        return None
                    return item
        query = _AppEngineUtilities_SessionData.all()
        query.filter(u"session = ", self)
        query.filter(u"keyname = ", keyname)
        results = query.fetch(1)
        if len(results) > 0:
            memcache.set(u"_AppEngineUtilities_SessionData_%s" % \
                (str(self.key())), self.get_items_ds())
            return results[0]
        return None

    def get_items_ds(self):
        """
        This gets all session data objects from the datastore, bypassing
        memcache.

        Returns a list of session data entities.
        """
        query = _AppEngineUtilities_SessionData.all()
        query.filter(u"session", self)
        results = query.fetch(1000)
        return results

    def delete(self):
        """
        Deletes a session and all it's associated data from the datastore and
        memcache.

        Returns True
        """
        try:
            query = _AppEngineUtilities_SessionData.all()
            query.filter(u"session = ", self)
            results = query.fetch(1000)
            db.delete(results)
            db.delete(self)
            memcache.delete_multi([u"_AppEngineUtilities_Session_%s" % \
                (str(self.key())), \
                u"_AppEngineUtilities_SessionData_%s" % \
                (str(self.key()))])
        except:
            mc = memcache.get(u"_AppEngineUtilities_Session_%s" %+ \
                (str(self.key())))
            if mc:
                mc.deleted = True
            else:
                # not in the memcache, check to see if it should be
                query = _AppEngineUtilities_Session.all()
                query.filter(u"sid = ", self.sid)
                results = query.fetch(1)
                if len(results) > 0:
                    results[0].deleted = True
                    memcache.set(u"_AppEngineUtilities_Session_%s" % \
                        (unicode(self.key())), results[0])
        return True
            
class _AppEngineUtilities_SessionData(db.Model):
    """
    Model for the session data in the datastore.
    """

    # session_key = db.FloatProperty()
    keyname = db.StringProperty()
    content = db.BlobProperty()
    model = db.ReferenceProperty()
    session = db.ReferenceProperty(_AppEngineUtilities_Session)
    dirty = db.BooleanProperty(default=False)
    deleted = db.BooleanProperty(default=False)

    def put(self):
        """
        Adds a keyname/value for session to the datastore and memcache

        Returns the key from the datastore put or u"dirty"
        """
        # update or insert in datastore
        try:
            return_val = db.put(self)
            self.dirty = False
        except:
            return_val = u"dirty"
            self.dirty = True

        # update or insert in memcache
        mc_items = memcache.get(u"_AppEngineUtilities_SessionData_%s" % \
            (str(self.session.key())))
        if mc_items:
            value_updated = False
            for item in mc_items:
                if value_updated == True:
                    break
                if item.keyname == self.keyname:
                    item.content = self.content
                    item.model = self.model
                    memcache.set(u"_AppEngineUtilities_SessionData_%s" % \
                        (str(self.session.key())), mc_items)
                    value_updated = True
                    break
            if value_updated == False:
                mc_items.append(self)
                memcache.set(u"_AppEngineUtilities_SessionData_%s" % \
                    (str(self.session.key())), mc_items)
        return return_val

    def delete(self):
        """
        Deletes an entity from the session in memcache and the datastore

        Returns True
        """
        try:
            db.delete(self)
        except:
            self.deleted = True
        mc_items = memcache.get(u"_AppEngineUtilities_SessionData_%s" % \
            (str(self.session.key())))
        value_handled = False
        for item in mc_items:
            if value_handled == True:
                break
            if item.keyname == self.keyname:
                if self.deleted == True:
                    item.deleted = True
                else:
                    mc_items.remove(item)
                memcache.set(u"_AppEngineUtilities_SessionData_%s" % \
                    (str(self.session.key())), mc_items)
        return True
        

class _DatastoreWriter(object):

    def put(self, keyname, value, session):
        """
        Insert a keyname/value pair into the datastore for the session.

        Args:
            keyname: The keyname of the mapping.
            value: The value of the mapping.

        Returns the model entity key
        """
        keyname = session._validate_key(keyname)
        if value is None:
            raise ValueError(u"You must pass a value to put.")

        # datestore write trumps cookie. If there is a cookie value
        # with this keyname, delete it so we don't have conflicting
        # entries.
        if session.cookie_vals.has_key(keyname):
            del(session.cookie_vals[keyname])
            session.output_cookie["%s_data" % (session.cookie_name)] = \
                simplejson.dumps(session.cookie_vals)
            session.output_cookie["%s_data" % (session.cookie_name)]["path"] = \
                session.cookie_path
            if session.cookie_domain:
                session.output_cookie["%s_data" % \
                    (session.cookie_name)]["domain"] = session.cookie_domain
            print session.output_cookie.output()

        sessdata = session._get(keyname=keyname)
        if sessdata is None:
            sessdata = _AppEngineUtilities_SessionData()
            # sessdata.session_key = session.session.key()
            sessdata.keyname = keyname
        try:
            db.model_to_protobuf(value)
            if not value.is_saved():
                value.put()
            sessdata.model = value
        except:
            sessdata.content = pickle.dumps(value)
            sessdata.model = None
        sessdata.session = session.session
            
        session.cache[keyname] = value
        return sessdata.put()


class _CookieWriter(object):
    def put(self, keyname, value, session):
        """
        Insert a keyname/value pair into the datastore for the session.

        Args:
            keyname: The keyname of the mapping.
            value: The value of the mapping.

        Returns True
        """
        keyname = session._validate_key(keyname)
        if value is None:
            raise ValueError(u"You must pass a value to put.")

        # Use simplejson for cookies instead of pickle.
        session.cookie_vals[keyname] = value
        # update the requests session cache as well.
        session.cache[keyname] = value
        # simplejson will raise any error I'd raise about an invalid value
        # so let it raise exceptions
        session.output_cookie["%s_data" % (session.cookie_name)] = \
            simplejson.dumps(session.cookie_vals)
        session.output_cookie["%s_data" % (session.cookie_name)]["path"] = \
            session.cookie_path
        if session.cookie_domain:
            session.output_cookie["%s_data" % \
                (session.cookie_name)]["domain"] = session.cookie_domain
        print session.output_cookie.output()
        return True

class Session(object):
    """
    Sessions are used to maintain user presence between requests.

    Sessions can either be stored server side in the datastore/memcache, or
    be kept entirely as cookies. This is set either with the settings file
    or on initialization, using the writer argument/setting field. Valid
    values are "datastore" or "cookie".

    Session can be used as a standard dictionary object.
        session = appengine_utilities.sessions.Session()
        session["keyname"] = "value" # sets keyname to value
        print session["keyname"] # will print value

    Datastore Writer:
        The datastore writer was written with the focus being on security,
        reliability, and performance. In that order.

        It is based off of a session token system. All data is stored
        server side in the datastore and memcache. A token is given to
        the browser, and stored server side. Optionally (and on by default),
        user agent and ip checking is enabled. Tokens have a configurable
        time to live (TTL), which defaults to 5 seconds. The current token,
        plus the previous 2, are valid for any request. This is done in order
        to manage ajax enabled sites which may have more than on request
        happening at a time. This means any token is valid for 15 seconds.
        A request with a token who's TTL has passed will have a new token
        generated.

        In order to take advantage of the token system for an authentication
        system, you will want to tie sessions to accounts, and make sure
        only one session is valid for an account. You can do this by setting
        a db.ReferenceProperty(_AppEngineUtilities_Session) attribute on
        your user Model, and use the get_ds_entity() method on a valid
        session to populate it on login.

        Note that even with this complex system, sessions can still be hijacked
        and it will take the user logging in to retrieve the account. In the
        future an ssl only cookie option may be implemented for the datastore
        writer, which would further protect the session token from being
        sniffed, however it would be restricted to using cookies on the
        .appspot.com domain, and ssl requests are a finite resource. This is
        why such a thing is not currently implemented.

        Session data objects are stored in the datastore pickled, so any
        python object is valid for storage.

    Cookie Writer:
        Sessions using the cookie writer are stored entirely in the browser
        and no interaction with the datastore is required. This creates
        a drastic improvement in performance, but provides no security for
        session hijack. This is useful for requests where identity is not
        important, but you wish to keep state between requests.

        Information is stored in a json format, as pickled data from the
        server is unreliable.

        Note: There is no checksum validation of session data on this method,
        it's streamlined for pure performance. If you need to make sure data
        is not tampered with, use the datastore writer which stores the data
        server side.

    django-middleware:
        Included with the GAEUtilties project is a
        django-middleware.middleware.SessionMiddleware which can be included in
        your settings file. This uses the cookie writer for anonymous requests,
        and you can switch to the datastore writer on user login. This will
        require an extra set in your login process of calling
        request.session.save() once you validated the user information. This
        will convert the cookie writer based session to a datastore writer.
    """

    # cookie name declaration for class methods
    COOKIE_NAME = settings.session["COOKIE_NAME"]

    def __init__(self, cookie_path=settings.session["DEFAULT_COOKIE_PATH"],
            cookie_domain=settings.session["DEFAULT_COOKIE_DOMAIN"],
            cookie_name=settings.session["COOKIE_NAME"],
            session_expire_time=settings.session["SESSION_EXPIRE_TIME"],
            clean_check_percent=settings.session["CLEAN_CHECK_PERCENT"],
            integrate_flash=settings.session["INTEGRATE_FLASH"],
            check_ip=settings.session["CHECK_IP"],
            check_user_agent=settings.session["CHECK_USER_AGENT"],
            set_cookie_expires=settings.session["SET_COOKIE_EXPIRES"],
            session_token_ttl=settings.session["SESSION_TOKEN_TTL"],
            last_activity_update=settings.session["UPDATE_LAST_ACTIVITY"],
            writer=settings.session["WRITER"]):
        """
        Initializer

        Args:
          cookie_path: The path setting for the cookie.
          cookie_domain: The domain setting for the cookie. (Set to False
                        to not use)
          cookie_name: The name for the session cookie stored in the browser.
          session_expire_time: The amount of time between requests before the
              session expires.
          clean_check_percent: The percentage of requests the will fire off a
              cleaning routine that deletes stale session data.
          integrate_flash: If appengine-utilities flash utility should be
              integrated into the session object.
          check_ip: If browser IP should be used for session validation
          check_user_agent: If the browser user agent should be used for
              sessoin validation.
          set_cookie_expires: True adds an expires field to the cookie so
              it saves even if the browser is closed.
          session_token_ttl: Number of sessions a session token is valid
              for before it should be regenerated.
        """

        self.cookie_path = cookie_path
        self.cookie_domain = cookie_domain
        self.cookie_name = cookie_name
        self.session_expire_time = session_expire_time
        self.integrate_flash = integrate_flash
        self.check_user_agent = check_user_agent
        self.check_ip = check_ip
        self.set_cookie_expires = set_cookie_expires
        self.session_token_ttl = session_token_ttl
        self.last_activity_update = last_activity_update
        self.writer = writer

        # make sure the page is not cached in the browser
        print self.no_cache_headers()
        # Check the cookie and, if necessary, create a new one.
        self.cache = {}
        string_cookie = os.environ.get(u"HTTP_COOKIE", u"")
        self.cookie = Cookie.SimpleCookie()
        self.output_cookie = Cookie.SimpleCookie()
        if string_cookie == "":
          self.cookie_vals = {}
        else:
            self.cookie.load(string_cookie)
            try:
                self.cookie_vals = \
                    simplejson.loads(self.cookie["%s_data" % (self.cookie_name)].value)
                    # sync self.cache and self.cookie_vals which will make those
                    # values available for all gets immediately.
                for k in self.cookie_vals:
                    self.cache[k] = self.cookie_vals[k]
                    # sync the input cookie with the output cookie
                    self.output_cookie["%s_data" % (self.cookie_name)] = \
                        simplejson.dumps(self.cookie_vals) #self.cookie["%s_data" % (self.cookie_name)]
            except Exception, e:
                self.cookie_vals = {}


        if writer == "cookie":
            pass
        else:
            self.sid = None
            new_session = True

            # do_put is used to determine if a datastore write should
            # happen on this request.
            do_put = False

            # check for existing cookie
            if self.cookie.get(cookie_name):
                self.sid = self.cookie[cookie_name].value
                # The following will return None if the sid has expired.
                self.session = _AppEngineUtilities_Session.get_session(self)
                if self.session:
                    new_session = False

            if new_session:
                # start a new session
                self.session = _AppEngineUtilities_Session()
                self.session.put()
                self.sid = self.new_sid()
                if u"HTTP_USER_AGENT" in os.environ:
                    self.session.ua = os.environ[u"HTTP_USER_AGENT"]
                else:
                    self.session.ua = None
                if u"REMOTE_ADDR" in os.environ:
                    self.session.ip = os.environ["REMOTE_ADDR"]
                else:
                    self.session.ip = None
                self.session.sid = [self.sid]
                # do put() here to get the session key
                self.session.put()
            else:
                # check the age of the token to determine if a new one
                # is required
                duration = datetime.timedelta(seconds=self.session_token_ttl)
                session_age_limit = datetime.datetime.now() - duration
                if self.session.last_activity < session_age_limit:
                    self.sid = self.new_sid()
                    if len(self.session.sid) > 2:
                        self.session.sid.remove(self.session.sid[0])
                    self.session.sid.append(self.sid)
                    do_put = True
                else:
                    self.sid = self.session.sid[-1]
                    # check if last_activity needs updated
                    ula = datetime.timedelta(seconds=self.last_activity_update)
                    if datetime.datetime.now() > self.session.last_activity + \
                        ula:
                        do_put = True

            self.output_cookie[cookie_name] = self.sid
            self.output_cookie[cookie_name]["path"] = self.cookie_path
            if self.cookie_domain:
                self.output_cookie[cookie_name]["domain"] = self.cookie_domain
            if self.set_cookie_expires:
                self.output_cookie[cookie_name]["expires"] = \
                    self.session_expire_time

            self.cache[u"sid"] = self.sid

            if do_put:
                if self.sid != None or self.sid != u"":
                    self.session.put()

        if self.set_cookie_expires:
            if not self.output_cookie.has_key("%s_data" % (cookie_name)):
                self.output_cookie["%s_data" % (cookie_name)] = u""
            self.output_cookie["%s_data" % (cookie_name)]["expires"] = \
                self.session_expire_time
        print self.output_cookie.output()

        # fire up a Flash object if integration is enabled
        if self.integrate_flash:
            import flash
            self.flash = flash.Flash(cookie=self.cookie)

        # randomly delete old stale sessions in the datastore (see
        # CLEAN_CHECK_PERCENT variable)
        if random.randint(1, 100) < clean_check_percent:
            self._clean_old_sessions() 

    def new_sid(self):
        """
        Create a new session id.

        Returns session id as a unicode string.
        """
        sid = u"%s_%s" % (str(self.session.key()),
            hashlib.md5(repr(time.time()) + \
            unicode(random.random())).hexdigest()
        )
        #sid = unicode(self.session.session_key) + "_" + \
        #        hashlib.md5(repr(time.time()) + \
        #        unicode(random.random())).hexdigest()
        return sid

    def _get(self, keyname=None):
        """
        private method
        
        Return all of the SessionData object data from the datastore only,
        unless keyname is specified, in which case only that instance of 
        SessionData is returned.

        Important: This does not interact with memcache and pulls directly
        from the datastore. This also does not get items from the cookie
        store.

        Args:
            keyname: The keyname of the value you are trying to retrieve.

        Returns a list of datastore entities.
        """
        if hasattr(self, 'session'):
            if keyname != None:
                return self.session.get_item(keyname)
            return self.session.get_items()
        return None
    
    def _validate_key(self, keyname):
        """
        private method
        
        Validate the keyname, making sure it is set and not a reserved name.

        Returns the validated keyname.
        """
        if keyname is None:
            raise ValueError(
                u"You must pass a keyname for the session data content."
            )
        elif keyname in (u"sid", u"flash"):
            raise ValueError(u"%s is a reserved keyname." % keyname)

        if type(keyname) != type([str, unicode]):
            return unicode(keyname)
        return keyname

    def _put(self, keyname, value):
        """
        Insert a keyname/value pair into the datastore for the session.

        Args:
            keyname: The keyname of the mapping.
            value: The value of the mapping.

        Returns the value from the writer put operation, varies based on writer.
        """
        if self.writer == "datastore":
            writer = _DatastoreWriter()
        else:
            writer = _CookieWriter()

        return writer.put(keyname, value, self)

    def _delete_session(self):
        """
        private method
        
        Delete the session and all session data.

        Returns True.
        """
        # if the event class has been loaded, fire off the preSessionDelete event
        if u"AEU_Events" in sys.modules['__main__'].__dict__:
            sys.modules['__main__'].AEU_Events.fire_event(u"preSessionDelete")
        if hasattr(self, u"session"):
            self.session.delete()
        self.cookie_vals = {}
        self.cache = {}
        self.output_cookie["%s_data" % (self.cookie_name)] = \
            simplejson.dumps(self.cookie_vals)
        self.output_cookie["%s_data" % (self.cookie_name)]["path"] = \
            self.cookie_path
        if self.cookie_domain:
            self.output_cookie["%s_data" % \
                (self.cookie_name)]["domain"] = self.cookie_domain

        print self.output_cookie.output()
        # if the event class has been loaded, fire off the sessionDelete event
        if u"AEU_Events" in sys.modules['__main__'].__dict__:
            sys.modules['__main__'].AEU_Events.fire_event(u"sessionDelete")
        return True

    def delete(self):
        """
        Delete the current session and start a new one.

        This is useful for when you need to get rid of all data tied to a
        current session, such as when you are logging out a user.

        Returns True
        """
        self._delete_session()

    @classmethod
    def delete_all_sessions(cls):
        """
        Deletes all sessions and session data from the data store. This
        does not delete the entities from memcache (yet). Depending on the
        amount of sessions active in your datastore, this request could
        timeout before completion and may have to be called multiple times.

        NOTE: This can not delete cookie only sessions as it has no way to
        access them. It will only delete datastore writer sessions.

        Returns True on completion.
        """
        all_sessions_deleted = False

        while not all_sessions_deleted:
            query = _AppEngineUtilities_Session.all()
            results = query.fetch(75)
            if len(results) is 0:
                all_sessions_deleted = True
            else:
                for result in results:
                    result.delete()
        return True


    def _clean_old_sessions(self):
        """
        Delete 50 expired sessions from the datastore.

        This is only called for CLEAN_CHECK_PERCENT percent of requests because
        it could be rather intensive.

        Returns True on completion
        """
        self.clean_old_sessions(self.session_expire_time, 50)


    @classmethod
    def clean_old_sessions(cls, session_expire_time, count=50):
        """
        Delete expired sessions from the datastore.

        This is a class method which can be used by applications for
        maintenance if they don't want to use the built in session
        cleaning.

        Args:
          count: The amount of session to clean.
          session_expire_time: The age in seconds to determine outdated
                               sessions.

        Returns True on completion
        """
        duration = datetime.timedelta(seconds=session_expire_time)
        session_age = datetime.datetime.now() - duration
        query = _AppEngineUtilities_Session.all()
        query.filter(u"last_activity <", session_age)
        results = query.fetch(50)
        for result in results:
            result.delete()
        return True

    def cycle_key(self):
        """
        Changes the session id/token.

        Returns new token.
        """
        self.sid = self.new_sid()
        if len(self.session.sid) > 2:
            self.session.sid.remove(self.session.sid[0])
        self.session.sid.append(self.sid)
        
        return self.sid

    def flush(self):
        """
        Delete's the current session, creating a new one.

        Returns True
        """
        self._delete_session()
        self.__init__()
        return True

    def no_cache_headers(self):
        """
        Generates headers to avoid any page caching in the browser.
        Useful for highly dynamic sites.

        Returns a unicode string of headers.
        """
        return u"".join([u"Expires: Tue, 03 Jul 2001 06:00:00 GMT",
            strftime("Last-Modified: %a, %d %b %y %H:%M:%S %Z").decode("utf-8"),
            u"Cache-Control: no-store, no-cache, must-revalidate, max-age=0",
            u"Cache-Control: post-check=0, pre-check=0",
            u"Pragma: no-cache",
        ])

    def clear(self):
        """
        Removes session data items, doesn't delete the session. It does work
        with cookie sessions, and must be called before any output is sent
        to the browser, as it set cookies.

        Returns True
        """
        sessiondata = self._get()
        # delete from datastore
        if sessiondata is not None:
            for sd in sessiondata:
                sd.delete()
        # delete from memcache
        self.cache = {}
        self.cookie_vals = {}
        self.output_cookie["%s_data" %s (self.cookie_name)] = \
            simplejson.dumps(self.cookie_vals)
        self.output_cookie["%s_data" % (self.cookie_name)]["path"] = \
            self.cookie_path
        if self.cookie_domain:
            self.output_cookie["%s_data" % \
                (self.cookie_name)]["domain"] = self.cookie_domain

        print self.output_cookie.output()
        return True

    def has_key(self, keyname):
        """
        Equivalent to k in a, use that form in new code

        Args:
            keyname: keyname to check

        Returns True/False
        """
        return self.__contains__(keyname)

    def items(self):
        """
        Creates a copy of just the data items.

        Returns dictionary of session data objects.
        """
        op = {}
        for k in self:
            op[k] = self[k]
        return op

    def keys(self):
        """
        Returns a list of keys.
        """
        l = []
        for k in self:
            l.append(k)
        return l

    def update(self, *dicts):
        """
        Updates with key/value pairs from b, overwriting existing keys

        Returns None
        """
        for dict in dicts:
            for k in dict:
                self._put(k, dict[k])
        return None

    def values(self):
        """
        Returns a list object of just values in the session.
        """
        v = []
        for k in self:
            v.append(self[k])
        return v

    def get(self, keyname, default = None):
        """
        Returns either the value for the keyname or a default value
        passed.

        Args:
            keyname: keyname to look up
            default: (optional) value to return on keyname miss

        Returns value of keyname, or default, or None
        """
        try:
            return self.__getitem__(keyname)
        except KeyError:
            if default is not None:
                return default
            return None

    def setdefault(self, keyname, default = None):
        """
        Returns either the value for the keyname or a default value
        passed. If keyname lookup is a miss, the keyname is set with
        a value of default.

        Args:
            keyname: keyname to look up
            default: (optional) value to return on keyname miss

        Returns value of keyname, or default, or None
        """
        try:
            return self.__getitem__(keyname)
        except KeyError:
            if default is not None:
                self.__setitem__(keyname, default)
                return default
            return None

    @classmethod
    def check_token(cls, cookie_name=COOKIE_NAME, delete_invalid=True):
        """
        Retrieves the token from a cookie and validates that it is
        a valid token for an existing cookie. Cookie validation is based
        on the token existing on a session that has not expired.

        This is useful for determining if datastore or cookie writer
        should be used in hybrid implementations.

        Args:
            cookie_name: Name of the cookie to check for a token.
            delete_invalid: If the token is not valid, delete the session
                            cookie, to avoid datastore queries on future
                            requests.

        Returns True/False
        """

        string_cookie = os.environ.get(u"HTTP_COOKIE", u"")
        cookie = Cookie.SimpleCookie()
        cookie.load(string_cookie)
        if cookie.has_key(cookie_name):
            query = _AppEngineUtilities_Session.all()
            query.filter(u"sid", cookie[cookie_name].value)
            results = query.fetch(1)
            if len(results) > 0:
                return True
            else:
                if delete_invalid:
                    output_cookie = Cookie.SimpleCookie()
                    output_cookie[cookie_name] = cookie[cookie_name]
                    output_cookie[cookie_name][u"expires"] = 0
                    print output_cookie.output()
        return False

    def get_ds_entity(self):
        """
        Will return the session entity from the datastore if one
        exists, otherwise will return None (as in the case of cookie writer
        session.
        """
        if hasattr(self, u"session"):
            return self.session
        return None

    # Implement Python container methods

    def __getitem__(self, keyname):
        """
        Get item from session data.

        keyname: The keyname of the mapping.
        """
        # flash messages don't go in the datastore

        if self.integrate_flash and (keyname == u"flash"):
            return self.flash.msg
        if keyname in self.cache:
            return self.cache[keyname]
        if keyname in self.cookie_vals:
            return self.cookie_vals[keyname]
        if hasattr(self, u"session"):
            data = self._get(keyname)
            if data:
                # TODO: It's broke here, but I'm not sure why, it's
                # returning a model object, but I can't seem to modify
                # it.
                try:
                    if data.model != None:
                        self.cache[keyname] = data.model
                        return self.cache[keyname]
                    else:
                        self.cache[keyname] = pickle.loads(data.content)
                        return self.cache[keyname]
                except:
                    self.delete_item(keyname)

            else:
                raise KeyError(unicode(keyname))
        raise KeyError(unicode(keyname))

    def __setitem__(self, keyname, value):
        """
        Set item in session data.

        Args:
            keyname: They keyname of the mapping.
            value: The value of mapping.
        """

        if self.integrate_flash and (keyname == u"flash"):
            self.flash.msg = value
        else:
            keyname = self._validate_key(keyname)
            self.cache[keyname] = value
            return self._put(keyname, value)

    def delete_item(self, keyname, throw_exception=False):
        """
        Delete item from session data, ignoring exceptions if
        necessary.

        Args:
            keyname: The keyname of the object to delete.
            throw_exception: false if exceptions are to be ignored.
        Returns:
            Nothing.
        """
        if throw_exception:
            self.__delitem__(keyname)
            return None
        else:
            try:
                self.__delitem__(keyname)
            except KeyError:
                return None

    def __delitem__(self, keyname):
        """
        Delete item from session data.

        Args:
            keyname: The keyname of the object to delete.
        """
        bad_key = False
        sessdata = self._get(keyname = keyname)
        if sessdata is None:
            bad_key = True
        else:
            sessdata.delete()
        if keyname in self.cookie_vals:
            del self.cookie_vals[keyname]
            bad_key = False
            self.output_cookie["%s_data" % (self.cookie_name)] = \
                simplejson.dumps(self.cookie_vals)
            self.output_cookie["%s_data" % (self.cookie_name)]["path"] = \
                self.cookie_path
            if self.cookie_domain:
                self.output_cookie["%s_data" % \
                    (self.cookie_name)]["domain"] = self.cookie_domain

            print self.output_cookie.output()
        if bad_key:
            raise KeyError(unicode(keyname))
        if keyname in self.cache:
            del self.cache[keyname]

    def __len__(self):
        """
        Return size of session.
        """
        # check memcache first
        if hasattr(self, u"session"):
            results = self._get()
            if results is not None:
                return len(results) + len(self.cookie_vals)
            else:
                return 0
        return len(self.cookie_vals)

    def __contains__(self, keyname):
        """
        Check if an item is in the session data.

        Args:
            keyname: The keyname being searched.
        """
        try:
            self.__getitem__(keyname)
        except KeyError:
            return False
        return True

    def __iter__(self):
        """
        Iterate over the keys in the session data.
        """
        # try memcache first
        if hasattr(self, u"session"):
            vals = self._get()
            if vals is not None:
                for k in vals:
                    yield k.keyname
        for k in self.cookie_vals:
            yield k

    def __str__(self):
        """
        Return string representation.
        """
        return u"{%s}" % ', '.join(['"%s" = "%s"' % (k, self[k]) for k in self])



#!/usr/bin/env python
#
# Copyright 2009 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Contains Unit Tests for Google Profiles API.

  ProfilesServiceTest: Provides methods to test feeds and manipulate items.

  ProfilesQueryTest: Constructs a query object for the profiles feed.
                 Extends Query.
"""

__author__ = 'jtoledo (Julian Toledo)'


import getopt
import getpass
import sys
import unittest
import gdata.contacts
import gdata.contacts.service


email = ''
password = ''
domain = ''
server = 'www.google.com'

GDATA_VER_HEADER = 'GData-Version'

class ProfilesServiceTest(unittest.TestCase):

  def setUp(self):    
    additional_headers = {GDATA_VER_HEADER: 3}
    self.gd_client = gdata.contacts.service.ContactsService(
        contact_list=domain, additional_headers=additional_headers )
    self.gd_client.email = email
    self.gd_client.password = password
    self.gd_client.source = 'GoogleInc-ProfilesPythonTest-1'
    self.gd_client.ProgrammaticLogin()

  def testGetFeedUriCustom(self):
    uri = self.gd_client.GetFeedUri(kind='profiles', scheme='https')
    self.assertEquals(
        'https://%s/m8/feeds/profiles/domain/%s/full' % (server, domain), uri)

  def testGetProfileFeedUriDefault(self):
    self.gd_client.contact_list = 'domain.com'
    self.assertEquals('/m8/feeds/profiles/domain/domain.com/full',
                      self.gd_client.GetFeedUri('profiles'))

  def testCleanUriNeedsCleaning(self):
    self.assertEquals('/relative/uri', self.gd_client._CleanUri(
        'http://www.google.com/relative/uri'))

  def testCleanUriDoesNotNeedCleaning(self):
    self.assertEquals('/relative/uri', self.gd_client._CleanUri(
        '/relative/uri'))

  def testGetProfilesFeed(self):
    feed = self.gd_client.GetProfilesFeed()
    self.assert_(isinstance(feed, gdata.contacts.ProfilesFeed))

  def testGetProfile(self):
    # Gets an existing entry
    feed = self.gd_client.GetProfilesFeed()
    entry = feed.entry[0]
    self.assert_(isinstance(entry, gdata.contacts.ProfileEntry))
    self.assertEquals(entry.title.text,
                      self.gd_client.GetProfile(entry.id.text).title.text)
    self.assertEquals(entry._children,
                      self.gd_client.GetProfile(entry.id.text)._children)

  def testUpdateProfile(self):
    feed = self.gd_client.GetProfilesFeed()
    entry = feed.entry[1]
    original_occupation = entry.occupation
    entry.occupation = gdata.contacts.Occupation(text='TEST')
    updated = self.gd_client.UpdateProfile(entry.GetEditLink().href, entry)
    self.assertEquals('TEST', updated.occupation.text)
    updated.occupation = original_occupation
    self.gd_client.UpdateProfile(updated.GetEditLink().href, updated)


if __name__ == '__main__':

  try:
    opts, args = getopt.getopt(sys.argv[1:], '', ['user=', 'pw=', 'domain='])
  except getopt.error, msg:
    print ('Profiles Tests\nNOTE: Please run these tests only with a test '
           'account. The tests may delete or update your data.\n'
           '\nUsage: service_test.py --email=EMAIL '
           '--password=PASSWORD --domain=DOMAIN\n')
    sys.exit(2)

  # Process options
  for option, arg in opts:
    if option == '--email':
      email = arg
    elif option == '--pw':
      password = arg
    elif option == '--domain':
      domain = arg

  while not email:
    print 'NOTE: Please run these tests only with a test account.'
    email = raw_input('Please enter your email: ')
  while not password:
    password = getpass.getpass('Please enter password: ')
    if not password:
      print 'Password cannot be blank.'
  while not domain:
    print 'NOTE: Please run these tests only with a test account.'
    domain = raw_input('Please enter your Apps domain: ')

  suite = unittest.makeSuite(ProfilesServiceTest)
  unittest.TextTestRunner().run(suite)




#!/usr/bin/python
#
# Copyright 2009 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


__author__ = 'e.bidelman (Eric Bidelman)'

import getopt
import mimetypes
import os.path
import sys

import gdata.sample_util
import gdata.sites.client
import gdata.sites.data


SOURCE_APP_NAME = 'googleInc-GoogleSitesAPIPythonLibSample-v1.1'
MAIN_MENU = ['1)  List site content',
             '2)  List recent activity',
             '3)  List revision history',
             '4)  Create webpage',
             '5)  Create web attachment',
             '6)  Upload attachment',
             '7)  Download attachment',
             '8)  Delete item',
             '9)  List sites',
             '10)  Create a new site',
             "11) List site's sharing permissions",
             '12) Change settings',
             '13) Exit']
SETTINGS_MENU = ['1) Change current site.',
                 '2) Change domain.']


class SitesExample(object):
  """Wrapper around the Sites API functionality."""

  def __init__(self, site_name=None, site_domain=None, debug=False):
    if site_domain is None:
      site_domain = self.PromptDomain()

    if site_name is None:
      site_name = self.PromptSiteName()

    mimetypes.init()

    self.client = gdata.sites.client.SitesClient(
        source=SOURCE_APP_NAME, site=site_name, domain=site_domain)
    self.client.http_client.debug = debug

    try:
      gdata.sample_util.authorize_client(
          self.client, service=self.client.auth_service, source=SOURCE_APP_NAME,
          scopes=['http://sites.google.com/feeds/',
                  'https://sites.google.com/feeds/'])
    except gdata.client.BadAuthentication:
      exit('Invalid user credentials given.')
    except gdata.client.Error:
      exit('Login Error')

  def PrintMainMenu(self):
    """Displays a menu of options for the user to choose from."""
    print '\nSites API Sample'
    print '================================'
    print '\n'.join(MAIN_MENU)
    print '================================\n'

  def PrintSettingsMenu(self):
    """Displays a menu of settings for the user change."""
    print '\nSites API Sample > Settings'
    print '================================'
    print '\n'.join(SETTINGS_MENU)
    print '================================\n'

  def GetMenuChoice(self, menu):
    """Retrieves the menu selection from the user.

    Args:
      menu: list The menu to get a selection from.

    Returns:
      The integer of the menu item chosen by the user.
    """
    max_choice = len(menu)
    while True:
      user_input = raw_input(': ')

      try:
        num = int(user_input)
      except ValueError:
        continue

      if num <= max_choice and num > 0:
        return num

  def PromptSiteName(self):
    site_name = ''
    while not site_name:
      site_name = raw_input('site name: ')
      if not site_name:
        print 'Please enter the name of your Google Site.'
    return site_name

  def PromptDomain(self):
    return raw_input(('If your Site is hosted on a Google Apps domain, '
                      'enter it (e.g. example.com): ')) or 'site'

  def GetChoiceSelection(self, feed, message):
    for i, entry in enumerate(feed.entry):
      print '%d.) %s' % (i + 1, entry.title.text)
    choice = 0
    while not choice or not 0 <= choice <= len(feed.entry):
      choice = int(raw_input(message))
    print
    return choice

  def PrintEntry(self, entry):
    print '%s [%s]' % (entry.title.text, entry.Kind())
    if entry.page_name:
      print ' page name:\t%s' % entry.page_name.text
    if entry.content:
      print ' content\t%s...' % str(entry.content.html)[0:100]

  def PrintListItem(self, entry):
    print '%s [%s]' % (entry.title.text, entry.Kind())
    for col in entry.field:
      print ' %s %s\t%s' % (col.index, col.name, col.text)

  def PrintListPage(self, entry):
    print '%s [%s]' % (entry.title.text, entry.Kind())
    for col in entry.data.column:
      print ' %s %s' % (col.index, col.name)

  def PrintFileCabinetPage(self, entry):
    print '%s [%s]' % (entry.title.text, entry.Kind())
    print ' page name:\t%s' % entry.page_name.text
    print ' content\t%s...' % str(entry.content.html)[0:100]

  def PrintAttachment(self, entry):
    print '%s [%s]' % (entry.title.text, entry.Kind())
    if entry.summary is not None:
      print ' description:\t%s' % entry.summary.text
    print ' content\t%s, %s' % (entry.content.type, entry.content.src)

  def PrintWebAttachment(self, entry):
    print '%s [%s]' % (entry.title.text, entry.Kind())
    if entry.summary.text is not None:
      print ' description:\t%s' % entry.summary.text
    print ' content src\t%s' % entry.content.src

  def Run(self):
    """Executes the demo application."""

    try:
      while True:
        self.PrintMainMenu()
        choice = self.GetMenuChoice(MAIN_MENU)

        if choice == 1:
          kind_choice = raw_input('What kind (all|%s)?: ' % '|'.join(
              gdata.sites.data.SUPPORT_KINDS))
          if kind_choice in gdata.sites.data.SUPPORT_KINDS:
            uri = '%s?kind=%s' % (self.client.make_content_feed_uri(),
                                  kind_choice)
            feed = self.client.GetContentFeed(uri=uri)
          else:
            feed = self.client.GetContentFeed()

          print "\nFetching content feed of '%s'...\n" % self.client.site

          for entry in feed.entry:
            kind = entry.Kind()

            if kind == 'attachment':
              self.PrintAttachment(entry)
            elif kind == 'webattachment':
              self.PrintWebAttachment(entry)
            elif kind == 'filecabinet':
              self.PrintFileCabinetPage(entry)
            elif kind == 'listitem':
              self.PrintListItem(entry)
            elif kind == 'listpage':
              self.PrintListPage(entry)
            else:
              self.PrintEntry(entry)

            print ' revision:\t%s' % entry.revision.text
            print ' updated:\t%s' % entry.updated.text

            parent_link = entry.FindParentLink()
            if parent_link:
              print ' parent link:\t%s' % parent_link

            if entry.GetAlternateLink():
              print ' view in Sites:\t%s' % entry.GetAlternateLink().href

            if entry.feed_link:
              print ' feed of items:\t%s' % entry.feed_link.href

            if entry.IsDeleted():
              print ' deleted:\t%s' % entry.IsDeleted()

            if entry.in_reply_to:
              print ' in reply to:\t%s' % entry.in_reply_to.href

            print

        elif choice == 2:
          print "\nFetching activity feed of '%s'..." % self.client.site
          feed = self.client.GetActivityFeed()
          for entry in feed.entry:
            print '  %s [%s on %s]' % (entry.title.text, entry.Kind(),
                                       entry.updated.text)

        elif choice == 3:
          print "\nFetching content feed of '%s'...\n" % self.client.site

          feed = self.client.GetContentFeed()
          try:
            selection = self.GetChoiceSelection(
                feed, 'Select a page to fetch revisions for: ')
          except TypeError:
            continue
          except ValueError:
            continue

          feed = self.client.GetRevisionFeed(
              feed.entry[selection - 1].GetNodeId())
          for entry in feed.entry:
            print entry.title.text
            print '  new version on:\t%s' % entry.updated.text
            print '  view changes:\t%s' % entry.GetAlternateLink().href
            print '  current version:\t%s...' % str(entry.content.html)[0:100]
            print

        elif choice == 4:
          print "\nFetching content feed of '%s'...\n" % self.client.site

          feed = self.client.GetContentFeed()
          try:
            selection = self.GetChoiceSelection(
                feed, 'Select a parent to upload to (or hit ENTER for none): ')
          except ValueError:
            selection = None

          page_title = raw_input('Enter a page title: ')

          parent = None
          if selection is not None:
            parent = feed.entry[selection - 1]

          new_entry = self.client.CreatePage(
              'webpage', page_title, '<b>Your html content</b>',
              parent=parent)
          if new_entry.GetAlternateLink():
            print 'Created. View it at: %s' % new_entry.GetAlternateLink().href

        elif choice == 5:
          print "\nFetching filecabinets on '%s'...\n" % self.client.site

          uri = '%s?kind=%s' % (self.client.make_content_feed_uri(),
                                'filecabinet')
          feed = self.client.GetContentFeed(uri=uri)

          selection = self.GetChoiceSelection(
              feed, 'Select a filecabinet to create the web attachment on: ')

          url = raw_input('Enter the URL of the attachment: ')
          content_type = raw_input("Enter the attachment's mime type: ")
          title = raw_input('Enter a title for the web attachment: ')
          description = raw_input('Enter a description: ')

          parent_entry = None
          if selection is not None:
            parent_entry = feed.entry[selection - 1]

          self.client.CreateWebAttachment(url, content_type, title,
                                          parent_entry, description=description)
          print 'Created!'

        elif choice == 6:
          print "\nFetching filecainets on '%s'...\n" % self.client.site

          uri = '%s?kind=%s' % (self.client.make_content_feed_uri(),
                                'filecabinet')
          feed = self.client.GetContentFeed(uri=uri)

          selection = self.GetChoiceSelection(
              feed, 'Select a filecabinet to upload to: ')

          filepath = raw_input('Enter a filename: ')
          page_title = raw_input('Enter a title for the file: ')
          description = raw_input('Enter a description: ')

          filename = os.path.basename(filepath)
          file_ex = filename[filename.rfind('.'):]
          if not file_ex in mimetypes.types_map:
            content_type = raw_input(
                'Unrecognized file extension. Please enter the mime type: ')
          else:
            content_type = mimetypes.types_map[file_ex]

          entry = None
          if selection is not None:
            entry = feed.entry[selection - 1]

          new_entry = self.client.UploadAttachment(
              filepath, entry, content_type=content_type, title=page_title,
              description=description)
          print 'Uploaded. View it at: %s' % new_entry.GetAlternateLink().href

        elif choice == 7:
          print "\nFetching all attachments on '%s'...\n" % self.client.site

          uri = '%s?kind=%s' % (self.client.make_content_feed_uri(),
                                'attachment')
          feed = self.client.GetContentFeed(uri=uri)

          selection = self.GetChoiceSelection(
              feed, 'Select an attachment to download: ')

          filepath = raw_input('Save as: ')

          entry = None
          if selection is not None:
            entry = feed.entry[selection - 1]

          self.client.DownloadAttachment(entry, filepath)
          print 'Downloaded.'

        elif choice == 8:
          print "\nFetching content feed of '%s'...\n" % self.client.site

          feed = self.client.GetContentFeed()
          selection = self.GetChoiceSelection(feed, 'Select a page to delete: ')

          entry = None
          if selection is not None:
            entry = feed.entry[selection - 1]

          self.client.Delete(entry)
          print 'Removed!'

        elif choice == 9:
          print ('\nFetching your list of sites for domain: %s...\n'
                 % self.client.domain)

          feed = self.client.GetSiteFeed()
          for entry in feed.entry:
            print entry.title.text
            print '  site name: ' + entry.site_name.text
            if entry.summary.text:
              print '  summary: ' + entry.summary.text
            if entry.FindSourceLink():
              print '  copied from site: ' + entry.FindSourceLink()
            print '  acl feed: %s\n' % entry.FindAclLink()

        elif choice == 10:
          title = raw_input('Enter a title: ')
          summary = raw_input('Enter a description: ')
          theme = raw_input('Theme name (ex. "default"): ')

          new_entry = self.client.CreateSite(
              title, description=summary, theme=theme)
          print 'Site created! View it at: ' + new_entry.GetAlternateLink().href

        elif choice == 11:
          print "\nFetching acl permissions of '%s'...\n" % self.client.site

          feed = self.client.GetAclFeed()
          for entry in feed.entry:
            print '%s (%s) - %s' % (entry.scope.value, entry.scope.type,
                                    entry.role.value)

        elif choice == 12:
          self.PrintSettingsMenu()
          settings_choice = self.GetMenuChoice(SETTINGS_MENU)

          if settings_choice == 1:
            self.client.site = self.PromptSiteName()
          elif settings_choice == 2:
            self.client.domain = self.PromptDomain()

        elif choice == 13:
          print 'Later!\n'
          return

    except gdata.client.RequestError, error:
      print error
    except KeyboardInterrupt:
      return


def main():
  """The main function runs the SitesExample application."""

  print 'NOTE: Please run these tests only with a test account.\n'

  try:
    opts, args = getopt.getopt(sys.argv[1:], '',
                               ['site=', 'domain=', 'debug'])
  except getopt.error, msg:
    print """python sites_sample.py --site [sitename]
                                    --domain [domain or "site"]
                                    --debug [prints debug info if set]"""
    sys.exit(2)

  site = None
  domain = None
  debug = False

  for option, arg in opts:
    if option == '--site':
      site = arg
    elif option == '--domain':
      domain = arg
    elif option == '--debug':
      debug = True

  sample = SitesExample(site, domain, debug=debug)
  sample.Run()


if __name__ == '__main__':
  main()



#!/usr/bin/env python
#
# Copyright (C) 2009 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


# This module is used for version 2 of the Google Data APIs.
# This test may make an actual HTTP request.


__author__ = 'j.s@google.com (Jeff Scudder)'


import unittest
import StringIO
import os.path
import atom.mock_http_core
import atom.http_core


class EchoClientTest(unittest.TestCase):
  
  def test_echo_response(self):
    client = atom.mock_http_core.EchoHttpClient()
    # Send a bare-bones POST request.
    request = atom.http_core.HttpRequest(method='POST', 
        uri=atom.http_core.Uri(host='www.jeffscudder.com', path='/'))
    request.add_body_part('hello world!', 'text/plain')
    response = client.request(request)
    self.assert_(response.getheader('Echo-Host') == 'www.jeffscudder.com:None')
    self.assert_(response.getheader('Echo-Uri') == '/')
    self.assert_(response.getheader('Echo-Scheme') is None)
    self.assert_(response.getheader('Echo-Method') == 'POST')
    self.assert_(response.getheader('Content-Length') == str(len(
        'hello world!')))
    self.assert_(response.getheader('Content-Type') == 'text/plain')
    self.assert_(response.read() == 'hello world!')

    # Test a path of None should default to /
    request = atom.http_core.HttpRequest(method='POST', 
        uri=atom.http_core.Uri(host='www.jeffscudder.com', path=None))
    response = client.request(request)
    self.assert_(response.getheader('Echo-Host') == 'www.jeffscudder.com:None')
    self.assert_(response.getheader('Echo-Method') == 'POST')
    self.assert_(response.getheader('Echo-Uri') == '/')
    
    # Send a multipart request.
    request = atom.http_core.HttpRequest(method='POST',
        uri=atom.http_core.Uri(scheme='https', host='www.jeffscudder.com', 
                               port=8080, path='/multipart', 
                               query={'test': 'true', 'happy': 'yes'}), 
        headers={'Authorization':'Test xyzzy', 'Testing':'True'})
    request.add_body_part('start', 'text/plain')
    request.add_body_part(StringIO.StringIO('<html><body>hi</body></html>'),
                          'text/html', len('<html><body>hi</body></html>'))
    request.add_body_part('alert("Greetings!")', 'text/javascript')
    response = client.request(request)
    self.assert_(response.getheader('Echo-Host') == 'www.jeffscudder.com:8080')
    self.assert_(
        response.getheader('Echo-Uri') == '/multipart?test=true&happy=yes')
    self.assert_(response.getheader('Echo-Scheme') == 'https')
    self.assert_(response.getheader('Echo-Method') == 'POST')
    self.assert_(response.getheader('Content-Type') == (
        'multipart/related; boundary="%s"' % (atom.http_core.MIME_BOUNDARY,)))
    expected_body = ('Media multipart posting'
                     '\r\n--%s\r\n'
                     'Content-Type: text/plain\r\n\r\n'
                     'start'
                     '\r\n--%s\r\n'
                     'Content-Type: text/html\r\n\r\n'
                     '<html><body>hi</body></html>'
                     '\r\n--%s\r\n'
                     'Content-Type: text/javascript\r\n\r\n'
                     'alert("Greetings!")'
                     '\r\n--%s--') % (atom.http_core.MIME_BOUNDARY, 
        atom.http_core.MIME_BOUNDARY, atom.http_core.MIME_BOUNDARY, 
        atom.http_core.MIME_BOUNDARY,)
    self.assert_(response.read() == expected_body)
    self.assert_(response.getheader('Content-Length') == str(
        len(expected_body)))


class MockHttpClientTest(unittest.TestCase):

  def setUp(self):
    self.client = atom.mock_http_core.MockHttpClient()

  def test_respond_with_recording(self):
    request = atom.http_core.HttpRequest(method='GET')
    atom.http_core.parse_uri('http://www.google.com/').modify_request(request)
    self.client.add_response(request, 200, 'OK', body='Testing')
    response = self.client.request(request)
    self.assert_(response.status == 200)
    self.assert_(response.reason == 'OK')
    self.assert_(response.read() == 'Testing')

  def test_save_and_load_recordings(self):
    request = atom.http_core.HttpRequest(method='GET')
    atom.http_core.parse_uri('http://www.google.com/').modify_request(request)
    self.client.add_response(request, 200, 'OK', body='Testing')
    response = self.client.request(request)
    self.client._save_recordings('test_save_and_load_recordings')
    self.client._recordings = []
    try:
      response = self.client.request(request)
      self.fail('There should be no recording for this request.')
    except atom.mock_http_core.NoRecordingFound:
      pass
    self.client._load_recordings('test_save_and_load_recordings')
    response = self.client.request(request)
    self.assert_(response.status == 200)
    self.assert_(response.reason == 'OK')
    self.assert_(response.read() == 'Testing')

  def test_use_recordings(self):
    request = atom.http_core.HttpRequest(method='GET')
    atom.http_core.parse_uri('http://www.google.com/').modify_request(request)
    self.client._load_or_use_client('test_use_recordings', 
        atom.http_core.HttpClient())
    response = self.client.request(request)
    if self.client.real_client:
      self.client._save_recordings('test_use_recordings')
    self.assert_(response.status in (200, 302))
    self.assert_(response.reason in ('OK', 'Found'))
    self.assert_(response.getheader('server') == 'gws')
    body = response.read()
    self.assert_(body.startswith('<!doctype html>') or
                 body.startswith('<HTML>'))

  def test_match_request(self):
    x = atom.http_core.HttpRequest('http://example.com/', 'GET')
    y = atom.http_core.HttpRequest('http://example.com/', 'GET')
    self.assert_(atom.mock_http_core._match_request(x, y))
    y = atom.http_core.HttpRequest('http://example.com/', 'POST')
    self.assert_(not atom.mock_http_core._match_request(x, y))
    y = atom.http_core.HttpRequest('http://example.com/1', 'GET')
    self.assert_(not atom.mock_http_core._match_request(x, y))
    y = atom.http_core.HttpRequest('http://example.com/?gsessionid=1', 'GET')
    self.assert_(not atom.mock_http_core._match_request(x, y))
    y = atom.http_core.HttpRequest('http://example.com/?start_index=1', 'GET')
    self.assert_(atom.mock_http_core._match_request(x, y))
    x = atom.http_core.HttpRequest('http://example.com/?gsessionid=1', 'GET')
    y = atom.http_core.HttpRequest('http://example.com/?gsessionid=1', 'GET')
    self.assert_(atom.mock_http_core._match_request(x, y))
    y = atom.http_core.HttpRequest('http://example.com/?gsessionid=2', 'GET')
    self.assert_(not atom.mock_http_core._match_request(x, y))
    y = atom.http_core.HttpRequest('http://example.com/', 'GET')
    self.assert_(not atom.mock_http_core._match_request(x, y))

  def test_use_named_sessions(self):
    self.client._delete_recordings('mock_http_test.test_use_named_sessions')
    self.client.use_cached_session('mock_http_test.test_use_named_sessions',
                                   atom.mock_http_core.EchoHttpClient())
    request = atom.http_core.HttpRequest('http://example.com', 'GET')
    response = self.client.request(request)
    self.assertEqual(response.getheader('Echo-Method'), 'GET')
    self.assertEqual(response.getheader('Echo-Host'), 'example.com:None')
    # We will insert a Cache-Marker header to indicate that this is a
    # recorded response, but initially it should not be present. 
    self.assertEqual(response.getheader('Cache-Marker'), None)
    # Modify the recorded response to allow us to identify a cached result
    # from an echoed result. We need to be able to check to see if this
    # came from a recording.
    self.assert_('Cache-Marker' not in self.client._recordings[0][1]._headers)
    self.client._recordings[0][1]._headers['Cache-Marker'] = '1'
    self.assert_('Cache-Marker' in self.client._recordings[0][1]._headers)
    # Save the recorded responses.
    self.client.close_session()

    # Create a new client, and have it use the recorded session.
    client = atom.mock_http_core.MockHttpClient()
    client.use_cached_session('mock_http_test.test_use_named_sessions',
                              atom.mock_http_core.EchoHttpClient())
    # Make the same request, which should use the recorded result. 
    response = client.request(request)
    self.assertEqual(response.getheader('Echo-Method'), 'GET')
    self.assertEqual(response.getheader('Echo-Host'), 'example.com:None')
    # We should now see the cache marker since the response is replayed.
    self.assertEqual(response.getheader('Cache-Marker'), '1')


def suite():
  return unittest.TestSuite((unittest.makeSuite(MockHttpClientTest, 'test'),
                             unittest.makeSuite(EchoClientTest, 'test')))


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python2.4
#
# Copyright 2011 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Sample to demonstrate the Email Settings API's labels and filters creation.
   The sample creates labels and filters corresponding to festivals,
   which are some days ahead.
"""

__author__ = 'Gunjan Sharma <gunjansharma@google.com>'

from datetime import date
from datetime import datetime
import getopt
import random
import sys
import time
import gdata.apps.client
import gdata.apps.emailsettings.client


SCOPES = ['https://apps-apis.google.com/a/feeds/emailsettings/2.0/',
          'https://apps-apis.google.com/a/feeds/user/']
FESTIVALS_LIST = [
    {
        'name': 'Christmas',
        'date': '25/12',
        'tags': ['Merry Christmas', 'Happy Christmas',
                 'Happy X-mas', 'Merry X-mas']
    },
    {
        'name': 'Halloween',
        'date': '31/10',
        'tags': ['Happy Halloween', 'Trick or Treat', 'Guise']
    },
    {
        'name': 'Sankranti',
        'date': '14/01',
        'tags': ['Makar Sankranti', 'Happy Sankranti']
    },
    {
        'name': 'New Year',
        'date': '31/12',
        'tags': ['Happy New Year', 'New Year Best Wishes']
    }]
# Number of days left before a festival should be less than MAX_DAY_LEFT
# to be considered for filter creation.
MAX_DAYS_LEFT = 15
# Maximum retries to be done for exponential back-off
MAX_RETRIES = 6


class FestivalSettingsException(Exception):
  """Exception class for FestivalSettings to show appropriate error message."""

  def __init__(self, message):
    """Create new FestivalSettingsException with appropriate error message."""
    super(FestivalSettingsException, self).__init__(message)


class FestivalSettings(object):
  """Sample demonstrating how to create filter and label for domain's user."""

  def __init__(self, consumer_key, consumer_secret, domain):
    """Create a new FestivalSettings object configured for a domain.

    Args:
      consumer_key: [string] The consumerKey of the domain.
      consumer_secret: [string] The consumerSecret of the domain.
      domain: [string] The domain whose user's POP settings to be changed.
    """
    self.consumer_key = consumer_key
    self.consumer_secret = consumer_secret
    self.domain = domain

  def Authorize(self):
    """Asks the domain's admin to authorize.

    Access to the two APIs needs to be authorized,
    provisioning users and Gmail settings.
    The function creates clients for both APIs.
    """
    self.email_settings_client = (
        gdata.apps.emailsettings.client.EmailSettingsClient(
            domain=self.domain))
    self.provisioning_client = gdata.apps.client.AppsClient(domain=self.domain)
    request_token = self.email_settings_client.GetOAuthToken(
        SCOPES, None, self.consumer_key, consumer_secret=self.consumer_secret)
    print request_token.GenerateAuthorizationUrl()

    raw_input('Manually go to the above URL and authenticate.'
              'Press Return after authorization.')
    access_token = self.email_settings_client.GetAccessToken(request_token)
    self.email_settings_client.auth_token = access_token
    self.provisioning_client.auth_token = access_token

  def _CreateLabelsFiltersForUser(self, username):
    """Creates labels and filters for a domain's user.

    Args:
      username: [string] The user's username to create labels and filters for.
    """
    today = date.today()

    for festival in FESTIVALS_LIST:
      d = datetime.strptime(festival['date'], '%d/%m').date()
      d = d.replace(year=today.year)
      days_left = (d - today).days
      d = d.replace(year=today.year+1)
      days_left_next_year = (d - today).days
      if ((0 <= days_left <= MAX_DAYS_LEFT) or
          (0 <= days_left_next_year <= MAX_DAYS_LEFT)):
        self._CreateFestivalSettingsForUser(username, festival, 1)

  def _CreateFestivalSettingsForUser(self, username, festival, tries):
    """Creates the festival wishes labels and filters for a domain's user.

    Args:
      username: [string] The user's username to create labels and filters for.
      festival: [dictionary] The details for the festival.
      tries: Number of times the operation has been retried.
    """
    label_name = festival['name'] + ' Wishes'
    try:
      self.email_settings_client.CreateLabel(username=username,
                                             name=label_name)
      for tag in festival['tags']:
        self.email_settings_client.CreateFilter(username=username,
                                                has_the_word=tag,
                                                label=label_name)
    except gdata.client.RequestError, e:
      if e.status == 503 and tries < MAX_RETRIES:
        time.sleep(2 ^ tries + random.randint(0, 10))
        self._CreateFestivalSettingsForUser(username, festival, tries+1)

  def Run(self):
    """Handles the flow of the sample.
    Asks application user for whom to create the festival wishes
    labels and filters.
    """
    print 'Whom would you like to create labels for?'
    print ('1 - For all user in the domain.'
           '(WARNING: May take a long time depending on your domain size.)')
    print '2 - Single user'

    choice = raw_input('Enter your choice: ').strip()
    if choice.isdigit():
      choice = int(choice)

    if choice == 1:
      users = self.provisioning_client.RetrieveAllUsers()
      for user in users.entry:
        self._CreateLabelsFiltersForUser(user.login.user_name)
    elif choice == 2:
      username = raw_input('Enter a valid username: ')
      self._CreateLabelsFiltersForUser(username)
    else:
      print 'Invalid choice'
      return
    print 'Labels and Filters created successfully'


def PrintUsageString():
  """Prints the correct call for running the sample."""
  print ('python emailsettings_labels_filters.py'
         '--consumer_key [ConsumerKey] --consumer_secret [ConsumerSecret]'
         '--domain [domain]')


def main():
  """Runs the sample using an instance of FestivalSettings."""

  try:
    opts, args = getopt.getopt(sys.argv[1:], '', ['consumer_key=',
                                                  'consumer_secret=',
                                                  'domain='])
  except getopt.error, msg:
    PrintUsageString()
    sys.exit(1)

  consumer_key = ''
  consumer_secret = ''
  domain = ''
  for option, arg in opts:
    if option == '--consumer_key':
      consumer_key = arg
    elif option == '--consumer_secret':
      consumer_secret = arg
    elif option == '--domain':
      domain = arg

  if not (consumer_key and consumer_secret and domain):
    print 'Requires exactly three flags.'
    PrintUsageString()
    sys.exit(1)

  festival_wishes_labels = FestivalSettings(
      consumer_key, consumer_secret, domain)
  try:
    festival_wishes_labels.Authorize()
  except gdata.client.RequestError, e:
    if e.status == 400:
      raise FestivalSettingsException('Invalid consumer credentials')
    else:
      raise FestivalSettingsException('Unknown server error')
    sys.exit(1)

  try:
    festival_wishes_labels.Run()
  except gdata.client.RequestError, e:
    if e.status == 403:
      raise FestivalSettingsException('Invalid Domain')
    elif e.status == 400:
      raise FestivalSettingsException('Invalid username')
    else:
      print e.reason
      raise FestivalSettingsException('Unknown error')


if __name__ == '__main__':
  main()
